# http://developer.intel.com/software/products/compilers/flin/

import os
import sys

from cpuinfo import cpu
from fcompiler import FCompiler, dummy_fortran_file
from exec_command import find_executable

class IntelFCompiler(FCompiler):

    compiler_type = 'intel'
    version_pattern = r'Intel\(R\) Fortran Compiler for 32-bit '\
                      'applications, Version (?P<version>[^\s*]*)'

    for fc_exe in map(find_executable,['ifort','ifc']):
        if os.path.isfile(fc_exe):
            break

    executables = {
        'version_cmd'  : [fc_exe, "-FI -V -c %(fname)s.f -o %(fname)s.o" \
                          % {'fname':dummy_fortran_file()}],
        'compiler_f77' : [fc_exe,"-72","-w90","-w95"],
        'compiler_fix' : [fc_exe,"-FI"],
        'compiler_f90' : [fc_exe],
        'linker_so'    : [fc_exe,"-shared"],
        'archiver'     : ["ar", "-cr"],
        'ranlib'       : ["ranlib"]
        }

    pic_flags = ['-KPIC']
    module_dir_switch = '-module ' # Don't remove ending space!
    module_include_switch = '-I'

    def get_flags(self):
        opt = self.pic_flags + ["-cm"]
        return opt

    def get_flags_free(self):
        return ["-FR"]

    def get_flags_opt(self):
        return ['-O3','-unroll']

    def get_flags_arch(self):
        opt = []
        if cpu.has_fdiv_bug():
            opt.append('-fdiv_check')
        if cpu.has_f00f_bug():
            opt.append('-0f_check')
        if cpu.is_PentiumPro() or cpu.is_PentiumII():
            opt.extend(['-tpp6','-xi'])
        elif cpu.is_PentiumIII():
            opt.append('-tpp6')
        elif cpu.is_Pentium():
            opt.append('-tpp5')
        elif cpu.is_PentiumIV() or cpu.is_XEON():
            opt.extend(['-tpp7','-xW'])
        if cpu.has_mmx():
            opt.append('-xM')
        return opt

    def get_flags_linker_so(self):
        opt = FCompiler.get_flags_linker_so(self)
        v = self.get_version()
        if v and v >= '8.0':
            opt.append('-nofor_main')
        return opt

class IntelItaniumFCompiler(IntelFCompiler):
    compiler_type = 'intele'
    version_pattern = r'Intel\(R\) Fortran 90 Compiler Itanium\(TM\) Compiler'\
                      ' for the Itanium\(TM\)-based applications,'\
                      ' Version (?P<version>[^\s*]*)'

    for fc_exe in map(find_executable,['efort','efc','ifort']):
        if os.path.isfile(fc_exe):
            break

    executables = {
        'version_cmd'  : [fc_exe, "-FI -V -c %(fname)s.f -o %(fname)s.o" \
                          % {'fname':dummy_fortran_file()}],
        'compiler_f77' : [fc_exe,"-FI","-w90","-w95"],
        'compiler_fix' : [fc_exe,"-FI"],
        'compiler_f90' : [fc_exe],
        'linker_so'    : [fc_exe,"-shared"],
        'archiver'     : ["ar", "-cr"],
        'ranlib'       : ["ranlib"]
        }

class IntelVisualFCompiler(FCompiler):

    compiler_type = 'intelv'
    version_pattern = r'Intel\(R\) Fortran Compiler for 32-bit applications, '\
                      'Version (?P<version>[^\s*]*)'

    ar_exe = 'lib.exe'
    fc_exe = 'ifl'
    if sys.platform=='win32':
        from distutils.msvccompiler import MSVCCompiler
        ar_exe = MSVCCompiler().lib

    executables = {
        'version_cmd'  : [fc_exe, "-FI -V -c %(fname)s.f -o %(fname)s.o" \
                          % {'fname':dummy_fortran_file()}],
        'compiler_f77' : [fc_exe,"-FI","-w90","-w95"],
        'compiler_fix' : [fc_exe,"-FI","-4L72","-w"],
        'compiler_f90' : [fc_exe],
        'linker_so'    : [fc_exe,"-shared"],
        'archiver'     : [ar_exe, "/verbose", "/OUT:"],
        'ranlib'       : None
        }

    compile_switch = '/c '
    object_switch = '/Fo'     #No space after /Fo!
    library_switch = '/OUT:'  #No space after /OUT:!
    module_dir_switch = '/module:' #No space after /module:
    module_include_switch = '/I'

    def get_flags(self):
        opt = ['/nologo','/MD','/nbs','/Qlowercase','/us']
        return opt

    def get_flags_free(self):
        return ["-FR"]

    def get_flags_debug(self):
        return ['/4Yb','/d2']

    def get_flags_opt(self):
        return ['/O3','/Qip','/Qipo','/Qipo_obj']

    def get_flags_arch(self):
        opt = []
        if cpu.is_PentiumPro() or cpu.is_PentiumII():
            opt.extend(['/G6','/Qaxi'])
        elif cpu.is_PentiumIII():
            opt.extend(['/G6','/QaxK'])
        elif cpu.is_Pentium():
            opt.append('/G5')
        elif cpu.is_PentiumIV():
            opt.extend(['/G7','/QaxW'])
        if cpu.has_mmx():
            opt.append('/QaxM')
        return opt

class IntelItaniumVisualFCompiler(IntelVisualFCompiler):

    compiler_type = 'intelev'
    version_pattern = r'Intel\(R\) Fortran 90 Compiler Itanium\(TM\) Compiler'\
                      ' for the Itanium\(TM\)-based applications,'\
                      ' Version (?P<version>[^\s*]*)'

    fc_exe = 'efl' # XXX this is a wild guess
    ar_exe = IntelVisualFCompiler.ar_exe

    executables = {
        'version_cmd'  : [fc_exe, "-FI -V -c %(fname)s.f -o %(fname)s.o" \
                          % {'fname':dummy_fortran_file()}],
        'compiler_f77' : [fc_exe,"-FI","-w90","-w95"],
        'compiler_fix' : [fc_exe,"-FI","-4L72","-w"],
        'compiler_f90' : [fc_exe],
        'linker_so'    : [fc_exe,"-shared"],
        'archiver'     : [ar_exe, "/verbose", "/OUT:"],
        'ranlib'       : None
        }

if __name__ == '__main__':
    from distutils import log
    log.set_verbosity(2)
    from fcompiler import new_fcompiler
    compiler = new_fcompiler(compiler='intel')
    compiler.customize()
    print compiler.get_version()



#!/usr/bin/env python

"""generate cephes_doc.h from included_functions.html"""

import string

def parse(infile):
    d={}
    key=None
    val=''
    prev_line = ''
    for line in infile.readlines():
        if not string.strip(line):
            continue
        if line[0]=='<':
            if key and val:
                d[key]=string.strip(val)
                key,val=None,None
            if line[:4]=='<DT>':
                tok=string.split(line)
                tok=string.split(tok[-1],'(')
                key=tok[0]
            elif line[:4]=='<DD>' and key:
                prev_line = prev_line[4:]
                tok = string.split(prev_line,' = ')
                val=tok[0]+'='+line[4:]
        else:
            if val:
                val=val+line
        prev_line = line

    return d

if __name__=="__main__":
    d = parse(open("docs/included_functions.html",'r'))
    keys = d.keys()
    keys.sort()
    ofile=open("cephes_doc.h",'w')
    for key in keys:
        ofile.write('#define %s_doc "%s"\n'%(key,repr(d[key])[1:-1]))
    ofile.close()



from c_spec import common_base_converter
from c_spec import num_to_c_types
from Numeric import *
from types import *
import os


num_typecode = {}
num_typecode['c'] = 'PyArray_CHAR'
num_typecode['1'] = 'PyArray_SBYTE'
num_typecode['b'] = 'PyArray_UBYTE'
num_typecode['s'] = 'PyArray_SHORT'
num_typecode['w'] = 'PyArray_USHORT'
num_typecode['i'] = 'PyArray_INT' # PyArray_INT has troubles ?? What does this note mean ??
num_typecode['u'] = 'PyArray_UINT'
num_typecode['l'] = 'PyArray_LONG'
num_typecode['f'] = 'PyArray_FLOAT'
num_typecode['d'] = 'PyArray_DOUBLE'
num_typecode['F'] = 'PyArray_CFLOAT'
num_typecode['D'] = 'PyArray_CDOUBLE'

type_check_code = \
"""
class numpy_type_handler
{
public:
    void conversion_numpy_check_type(PyArrayObject* arr_obj, int numeric_type,
                                     const char* name)
    {
        // Make sure input has correct numeric type.
        // allow character and byte to match
        // also allow int and long to match
        int arr_type = arr_obj->descr->type_num;
        if ( arr_type != numeric_type &&
            !(numeric_type == PyArray_CHAR  && arr_type == PyArray_SBYTE) &&
            !(numeric_type == PyArray_SBYTE && arr_type == PyArray_CHAR)  &&
            !(numeric_type == PyArray_INT   && arr_type == PyArray_LONG)  &&
            !(numeric_type == PyArray_LONG  && arr_type == PyArray_INT)) 
        {

            char* type_names[20] = {"char","unsigned byte","byte", "short", "unsigned short",
                                    "int", "unsigned int", "long", "float", "double", 
                                    "complex float","complex double", "object","ntype",
                                    "unkown"};
            char msg[500];
            sprintf(msg,"Conversion Error: received '%s' typed array instead of '%s' typed array for variable '%s'",
                    type_names[arr_type],type_names[numeric_type],name);
            throw_error(PyExc_TypeError,msg);    
        }
    }
    
    void numpy_check_type(PyArrayObject* arr_obj, int numeric_type, const char* name)
    {
        // Make sure input has correct numeric type.
        int arr_type = arr_obj->descr->type_num;
        if ( arr_type != numeric_type &&
            !(numeric_type == PyArray_CHAR  && arr_type == PyArray_SBYTE) &&
            !(numeric_type == PyArray_SBYTE && arr_type == PyArray_CHAR)  &&
            !(numeric_type == PyArray_INT   && arr_type == PyArray_LONG)  &&
            !(numeric_type == PyArray_LONG  && arr_type == PyArray_INT)) 
        {
            char* type_names[20] = {"char","unsigned byte","byte", "short", 
                                    "unsigned short", "int", "unsigned int",
                                    "long", "float", "double", 
                                    "complex float", "complex double", 
                                    "object","ntype","unkown"};
            char msg[500];
            sprintf(msg,"received '%s' typed array instead of '%s' typed array for variable '%s'",
                    type_names[arr_type],type_names[numeric_type],name);
            throw_error(PyExc_TypeError,msg);    
        }
    }
};

numpy_type_handler x__numpy_type_handler = numpy_type_handler();
#define conversion_numpy_check_type x__numpy_type_handler.conversion_numpy_check_type
#define numpy_check_type x__numpy_type_handler.numpy_check_type

"""

size_check_code = \
"""
class numpy_size_handler
{
public:
    void conversion_numpy_check_size(PyArrayObject* arr_obj, int Ndims, 
                                     const char* name)
    {
        if (arr_obj->nd != Ndims)
        {
            char msg[500];
            sprintf(msg,"Conversion Error: received '%d' dimensional array instead of '%d' dimensional array for variable '%s'",
                    arr_obj->nd,Ndims,name);
            throw_error(PyExc_TypeError,msg);
        }    
    }
    
    void numpy_check_size(PyArrayObject* arr_obj, int Ndims, const char* name)
    {
        if (arr_obj->nd != Ndims)
        {
            char msg[500];
            sprintf(msg,"received '%d' dimensional array instead of '%d' dimensional array for variable '%s'",
                    arr_obj->nd,Ndims,name);
            throw_error(PyExc_TypeError,msg);
        }    
    }
};

numpy_size_handler x__numpy_size_handler = numpy_size_handler();
#define conversion_numpy_check_size x__numpy_size_handler.conversion_numpy_check_size
#define numpy_check_size x__numpy_size_handler.numpy_check_size

"""

numeric_init_code = \
"""
Py_Initialize();
import_array();
PyImport_ImportModule("Numeric");
"""
    
class array_converter(common_base_converter):

    def init_info(self):
        common_base_converter.init_info(self)
        self.type_name = 'numpy'
        self.check_func = 'PyArray_Check'    
        self.c_type = 'PyArrayObject*'
        self.return_type = 'PyArrayObject*'
        self.to_c_return = '(PyArrayObject*) py_obj'
        self.matching_types = [ArrayType]
        self.headers = ['"Numeric/arrayobject.h"','<complex>','<math.h>']
        self.support_code = [size_check_code, type_check_code]
        self.module_init_code = [numeric_init_code]    
               
    def get_var_type(self,value):
        return value.typecode()
    
    def template_vars(self,inline=0):
        res = common_base_converter.template_vars(self,inline)    
        if hasattr(self,'var_type'):
            res['num_type'] = num_to_c_types[self.var_type]
            res['num_typecode'] = num_typecode[self.var_type]
        res['array_name'] = self.name + "_array"
        return res
         
    def declaration_code(self,templatize = 0,inline=0):
        code = '%(py_var)s = %(var_lookup)s;\n'   \
               '%(c_type)s %(array_name)s = %(var_convert)s;\n'  \
               'conversion_numpy_check_type(%(array_name)s,%(num_typecode)s,"%(name)s");\n' \
               'int* N%(name)s = %(array_name)s->dimensions;\n' \
               'int* S%(name)s = %(array_name)s->strides;\n' \
               'int D%(name)s = %(array_name)s->nd;\n' \
               '%(num_type)s* %(name)s = (%(num_type)s*) %(array_name)s->data;\n' 
        code = code % self.template_vars(inline=inline)
        return code



# TNC Python interface
# @(#) $Jeannot: tnc.py,v 1.7 2004/04/02 20:40:21 js Exp $

# Copyright (c) 2004, Jean-Sebastien Roy (js@jeannot.org)

# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:

# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""
TNC: A python interface to the TNC non-linear optimizer

TNC is a non-linear optimizer. To use it, you must provide a function to
minimize. The function must take one argument: the list of coordinates where to
evaluate the function; and it must return either a tuple, whose first element is the
value of the function, and whose second argument is the gradient of the function
(as a list of values); or None, to abort the minimization.
"""

import moduleTNC
from scipy_base import asarray

MSG_NONE = 0 # No messages
MSG_ITER = 1 # One line per iteration
MSG_INFO = 2 # Informational messages
MSG_VERS = 4 # Version info
MSG_EXIT = 8 # Exit reasons
MSG_ALL = MSG_ITER + MSG_INFO + MSG_VERS + MSG_EXIT

MSGS = {
        MSG_NONE : "No messages",
        MSG_ITER : "One line per iteration",
        MSG_INFO : "Informational messages",
        MSG_VERS : "Version info",
        MSG_EXIT : "Exit reasons",
        MSG_ALL  : "All messages"
}

HUGE_VAL=1e500 # No standard representation of Infinity in Python 2.3.3

EINVAL       = -2 # Invalid parameters (n<1)
INFEASIBLE   = -1 # Infeasible (low > up)
LOCALMINIMUM =  0 # Local minima reach (|pg| ~= 0)
CONVERGED    =  1 # Converged (|f_n-f_(n-1)| ~= 0)
MAXFUN       =  2 # Max. number of function evaluations reach
LSFAIL       =  3 # Linear search failed
CONSTANT     =  4 # All lower bounds are equal to the upper bounds
NOPROGRESS   =  5 # Unable to progress
USERABORT    =  6 # User requested end of minimization

RCSTRINGS = {
        EINVAL       : "Invalid parameters (n<1)",
        INFEASIBLE   : "Infeasible (low > up)",
        LOCALMINIMUM : "Local minima reach (|pg| ~= 0)",
        CONVERGED    : "Converged (|f_n-f_(n-1)| ~= 0)",
        MAXFUN       : "Max. number of function evaluations reach",
        LSFAIL       : "Linear search failed",
        CONSTANT     : "All lower bounds are equal to the upper bounds",
        NOPROGRESS   : "Unable to progress",
        USERABORT    : "User requested end of minimization"
}


# Changes to interface made by Travis Oliphant, Apr. 2004 for inclusion in
#  SciPy

import optimize
approx_fprime = optimize.approx_fprime


def fmin_tnc(func, x0, fprime=None, args=(), approx_grad=False, bounds=None, epsilon=1e-8,
             scale=None, messages=MSG_ALL, maxCGit=-1, maxfun=None, eta=-1,
             stepmx=0, accuracy=0, fmin=0, ftol=0, rescale=-1):
    """Minimize a function with variables subject to bounds, using gradient 
    information.
    
    returns (rc, nfeval, x).
    
    Inputs:

    func    -- function to minimize. Called as func(x, *args)
    
    x0      -- initial guess to minimum
    
    fprime  -- gradient of func. If None, then func returns the function
               value and the gradient ( f, g = func(x, *args) ).
               Called as fprime(x, *args)
                   
    args    -- arguments to pass to function

    approx_grad -- if true, approximate the gradient numerically    

    bounds  -- a list of (min, max) pairs for each element in x, defining
               the bounds on that parameter. Use None for one of min or max
               when there is no bound in that direction

        scale     : scaling factors to apply to each variable (a list of floats)
                    if None, the factors are up-low for interval bounded variables
                    and 1+|x] fo the others.
                    defaults to None
        messages  : bit mask used to select messages display during minimization
                    values defined in the optimize.tnc.MSGS dict.
                    defaults to optimize.tnc.MGS_ALL
        maxCGit   : max. number of hessian*vector evaluation per main iteration
                    if maxCGit == 0, the direction chosen is -gradient
                    if maxCGit < 0, maxCGit is set to max(1,min(50,n/2))
                    defaults to -1
        maxnfeval : max. number of function evaluation
                    if None, maxnfeval is set to max(1000, 100*len(x0))
                    defaults to None
        eta       : severity of the line search. if < 0 or > 1, set to 0.25
                    defaults to -1
        stepmx    : maximum step for the line search. may be increased during call
                    if too small, will be set to 10.0
                    defaults to 0
        accuracy  : relative precision for finite difference calculations
                    if <= machine_precision, set to sqrt(machine_precision)
                    defaults to 0
        fmin      : minimum function value estimate
                    defaults to 0
        ftol      : precision goal for the value of f in the stoping criterion
                    relative to the machine precision and the value of f.
                    if ftol < 0.0, ftol is set to 0.0
                    defaults to 0
        rescale   : Scaling factor (in log10) used to trigger rescaling
                    if 0, rescale at each iteration
                    if a large value, never rescale
                    if < 0, rescale is set to 1.3

    Outputs:

        x         : the solution (a list of floats)
        nfeval    : the number of function evaluations
        rc        : return code (corresponding message in optimize.tnc.RCSTRINGS)
    """

    n = len(x0)

    if bounds is None:
        bounds = [(None,None)] * n
    if len(bounds) != n:
        raise ValueError('length of x0 != length of bounds')

    if approx_grad:
        def func_and_grad(x):
            x = asarray(x)
            f = func(x, *args)
            g = approx_fprime(x, func, epsilon, *args)
            return f, list(g)
    elif fprime is None:
        def func_and_grad(x):
            x = asarray(x)
            f, g = func(x, *args)
            return f, list(g)
    else:
        def func_and_grad(x):
            x = asarray(x)
            f = func(x, *args)
            g = fprime(x, *args)
            return f, list(g)

    low = [0]*n
    up = [0]*n
    for i in range(n):
        l,u = bounds[i]
        if l is None:
            low[i] = -HUGE_VAL
        else:
            low[i] = l
        if u is None:
            up[i] = HUGE_VAL
        else:
            up[i] = l
        
    if scale == None:
        scale = []

    if maxfun == None:
       maxfun = max(1000, 100*len(x0))
        
    return moduleTNC.minimize(func_and_grad, x0, low, up, scale, messages,
                              maxCGit, maxfun, eta, stepmx, accuracy,
                              fmin, ftol, rescale)

if __name__ == '__main__':
        # Examples for TNC

        def example():
                print "Example"
                # A function to minimize
                def function(x):
                        f = pow(x[0],2.0)+pow(abs(x[1]),3.0)
                        g = [0,0]
                        g[0] = 2.0*x[0]
                        g[1] = 3.0*pow(abs(x[1]),2.0)
                        if x[1]<0:
                                g[1] = -g[1]
                        return f, g

                # Optimizer call
                rc, nf, x = minimize(function, [-7, 3], bounds=([-10, 10], [1, 10]))

                print "After", nf, "function evaluations, TNC returned:", RCSTRINGS[rc]
                print "x =", x
                print "exact value = [0, 1]"
                print

        example()

        # Tests
        # These tests are taken from Prof. K. Schittkowski test examples for 
        # constrained nonlinear programming.
        # http://www.uni-bayreuth.de/departments/math/~kschittkowski/home.htm
        tests = []
        def test1fg(x):
                f = 100.0*pow((x[1]-pow(x[0],2)),2)+pow(1.0-x[0],2)
                dif = [0,0]
                dif[1] = 200.0*(x[1]-pow(x[0],2))
                dif[0] = -2.0*(x[0]*(dif[1]-1.0)+1.0)
                return f, dif
        tests.append ((test1fg, [-2,1], ([-HUGE_VAL,None],[-1.5,None]), [1,1]))

        def test2fg(x):
                f = 100.0*pow((x[1]-pow(x[0],2)),2)+pow(1.0-x[0],2)
                dif = [0,0]
                dif[1] = 200.0*(x[1]-pow(x[0],2))
                dif[0] = -2.0*(x[0]*(dif[1]-1.0)+1.0)
                return f, dif
        tests.append ((test2fg, [-2,1], [(-HUGE_VAL,None),(1.5,None)], [-1.2210262419616387,1.5]))

        def test3fg(x):
                f = x[1]+pow(x[1]-x[0],2)*1.0e-5
                dif = [0,0]
                dif[0] = -2.0*(x[1]-x[0])*1.0e-5
                dif[1] = 1.0-dif[0]
                return f, dif
        tests.append ((test3fg, [10,1], [(-HUGE_VAL,None),(0.0, None)], [0,0]))

        def test4fg(x):
                f = pow(x[0]+1.0,3)/3.0+x[1]
                dif = [0,0]
                dif[0] = pow(x[0]+1.0,2)
                dif[1] = 1.0
                return f, dif
        tests.append ((test4fg, [1.125,0.125], [(1, None),(0, None)], [1,0]))

        from math import *

        def test5fg(x):
                f = sin(x[0]+x[1])+pow(x[0]-x[1],2)-1.5*x[0]+2.5*x[1]+1.0
                dif = [0,0]
                v1 = cos(x[0]+x[1]);
                v2 = 2.0*(x[0]-x[1]);

                dif[0] = v1+v2-1.5;
                dif[1] = v1-v2+2.5;
                return f, dif
        tests.append ((test5fg, [0,0], [(-1.5, 4),(-3,3)], [-0.54719755119659763, -1.5471975511965976]))

        def test38fg(x):
                f = (100.0*pow(x[1]-pow(x[0],2),2)+pow(1.0-x[0],2)+90.0*pow(x[3]-pow(x[2],2),2) \
                        +pow(1.0-x[2],2)+10.1*(pow(x[1]-1.0,2)+pow(x[3]-1.0,2)) \
                        +19.8*(x[1]-1.0)*(x[3]-1.0))*1.0e-5
                dif = [0,0,0,0]
                dif[0] = (-400.0*x[0]*(x[1]-pow(x[0],2))-2.0*(1.0-x[0]))*1.0e-5
                dif[1] = (200.0*(x[1]-pow(x[0],2))+20.2*(x[1]-1.0)+19.8*(x[3]-1.0))*1.0e-5
                dif[2] = (-360.0*x[2]*(x[3]-pow(x[2],2))-2.0*(1.0-x[2]))*1.0e-5
                dif[3] = (180.0*(x[3]-pow(x[2],2))+20.2*(x[3]-1.0)+19.8*(x[1]-1.0))*1.0e-5
                return f, dif
        tests.append ((test38fg, [-3,-1,-3,-1], [(-10,10)]*4, [1]*4))

        def test45fg(x):
                f = 2.0-x[0]*x[1]*x[2]*x[3]*x[4]/120.0
                dif = [0]*5
                dif[0] = -x[1]*x[2]*x[3]*x[4]/120.0
                dif[1] = -x[0]*x[2]*x[3]*x[4]/120.0
                dif[2] = -x[0]*x[1]*x[3]*x[4]/120.0
                dif[3] = -x[0]*x[1]*x[2]*x[4]/120.0
                dif[4] = -x[0]*x[1]*x[2]*x[3]/120.0
                return f, dif
        tests.append ((test45fg, [2]*5, [(0,1),(0,2),(0,3),(0,4),(0,5)], [1,2,3,4,5]))

        def test(fg, x, bounds, xopt):
                print "** Test", fg.__name__
                rc, nf, x = minimize(fg, x, bounds=bounds,  messages = MSG_NONE, maxnfeval = 200)
                print "After", nf, "function evaluations, TNC returned:", RCSTRINGS[rc]
                print "x =", x
                print "exact value =", xopt
                enorm = 0.0
                norm = 1.0
                for y,yo in zip(x, xopt):
                        enorm += (y-yo)*(y-yo)
                        norm += yo*yo
                e = pow(enorm/norm, 0.5)
                print "Error =", e
                if e > 1e-8:
                        raise "Test "+fg.__name__+" failed"

        for fg, x, bounds, xopt in tests:
                test(fg, x, bounds, xopt)
        
        print
        print "** All TNC tests passed."



from scipy import ga

def fitness_ex1(self):
    score = 0
    desired = ['a','b','c','d','b',-1,.25,.6, .8, 0]
    # compare letters
    actual = self.get_values()
    pairs = zip(actual[:5],desired[:5])
    for g,d in pairs:
        if g == d:
            score += 1
    pairs = zip(actual[5:],desired[5:])
    for g,d in pairs:
        diff = g - d
        score -= abs(diff)
    return score        
    
def ex1():
    """ This example illustrates the main steps in setting
        up a genetic optimization:
        
          1. Specify the genes types used to encode your problem
          2. Group these genes into a genome.
             a.  Specify the fitness function that evaluates the genomes.
          3. Create a population of the genomes
          4. Specify the algorithm used to evolve the population
          5.  
    """
    
    # 1. First scpecify your genes.  To gene types are 
    #    currently supported, list_gene and float_gene
    
    #    A list gene chooses its value from a list of values.
    #    The list can contain any type of object.
    g1 = ga.gene.list_gene(['a','b','c','d'])    
    #    Float genes take on a continuous value between two
    #    bounds.
    g2 = ga.gene.float_gene((-1.,1.))    
    #    We'll replicate these genes several times to make a longer
    #    genome.
    all_genes = g1.replicate(5) + g2.replicate(5)    
    # 2.  Create a specialized "list_genome" (as opposed to tree_genome) 
    #     class with the desired fitness function.
    #     It's structure is defined by our gene list. 
    class this_genome(ga.genome.list_genome):
        pass
    this_genome.performance = fitness_ex1            
    gnm = this_genome(all_genes)

    # 3.  Create a population of the genomes.
    #
    pop = ga.population.population(gnm)
    # 4.  Now use the basic genetic algorithm to evolve the population
    #
    galg = ga.algorithm.galg(pop)
    # change a few settings
    settings = {'pop_size':250,'p_replace':.8,'p_cross': .8, 'p_mutate':'gene',
                'p_deviation': 0.,'gens':35,'rand_seed':0,'rand_alg':'CMRG'}
    galg.settings.update(settings)
    galg.evolve()
    print galg.pop.best()
    
if __name__ == '__main__':
    ex1()    


#!/usr/bin/env python

import os
from glob import glob
from misc_util import get_path, default_config_dict, dot_join

def configuration(parent_package='',parent_path=None):
    package = 'scipy_distutils'
    local_path = get_path(__name__,parent_path)
    config = default_config_dict(package,parent_package)

    sub_package = dot_join(parent_package,package,'command')
    config['packages'].append(sub_package)
    config['package_dir'][sub_package] = os.path.join(local_path,'command')

    #for name in glob(os.path.join(local_path,'tests','*_ext')):
    #    p = dot_join(parent_package,package,'tests',os.path.basename(name))
    #    config['packages'].append(p)
    #    config['package_dir'][p] = name

    config['data_files'].append((package,[os.path.join(local_path,'sample_site.cfg')]))
    return config

if __name__ == '__main__':
    from scipy_distutils_version import scipy_distutils_version
    print 'scipy_distutils Version',scipy_distutils_version
    from core import setup
    config = configuration(parent_path='')
    for k,v in config.items():
        if not v:
            del config[k]
    setup(version = scipy_distutils_version,
          description = "Changes to distutils needed for SciPy "\
          "-- mostly Fortran support",
          author = "Travis Oliphant, Eric Jones, and Pearu Peterson",
          author_email = "scipy-dev@scipy.org",
          license = "SciPy License (BSD Style)",
          url = 'http://www.scipy.org',
          **config
          )



#! /usr/bin/env python
# Copyright (c) 1996, 1997, The Regents of the University of California.
# All rights reserved.  See Legal.htm for full text and disclaimer.
#  $Id: gistdemolow.py,v 1.3 2003/09/30 23:43:47 travo Exp $
#  ---------------------------------------------------------------------
# 
#  NAME:    gistdemolow.py
#  CHANGES:
#  04/07/03 llc Add plh test.
#  03/14/03 llc Add test for plfp (test 23).
#  12/03/01 llc Correct titles for Test 12 and Test 18.  
#               Add pltitle for Test 12.
#
#  ---------------------------------------------------------------------

from gist import *
import time
from math import pi

def grtest():

  print "\n\n         Pygist Comprehensive Graphics Test\n"
  print "This test suite is similar, but not exactly the same as yorick's grtest\n"
  print "Each frame will be described at the terminal."
  print "Compare what you see with the description, then"
  print "hit <RETURN> to see the next test, or q <RETURN> to quit.\n"
  if quitnow(): return
  pldefault(marks=1, width=0, type=1, style="work.gs", dpi=100)
  winkill(0)
  print "Test 1:  Commands: window (0, wait=1, dpi=75); plg([0, 1])"
  window (0, wait=1, dpi=75)
  plg([0, 1])
  print "A small (75 dpi) window with line marked A from (1,0) to (2,1)."
  if quitnow(): return
  winkill(0)

  print "Test 2:  Commands: plg([0,1])"
  plg([0,1])
  print "A large (100 dpi) window with line marked A from (1,0) to (2,1)."
  if quitnow(): return
  unzoom()

  print "Test 3:  Commands: plg([1, 0])"
  plg([1, 0])
  print "Added line marked B from (1,1) to (2,0) to previous plot."
  if quitnow(): return
  unzoom()

  print "Test 4:  Commands: logxy(1, 0)"
  logxy(1, 0)
  print "X axis now a log scale."
  if quitnow(): return
  unzoom()

  print "Test 5:  Commands: logxy(0, 0)"
  logxy(0, 0)
  print "X axis back to linear scale."
  if quitnow(): return
  unzoom()

  print "Test 6:  Commands: limits(1.2, 1.8, 0.2, 0.8)"
  limits(1.2, 1.8, 0.2, 0.8)
  print "Limits changed to 1.2<x<1.8, 0.2<y<0.8."
  if quitnow(): return
  unzoom()

  ylimits(0.4, 0.6)
  print "Test 7:  Commands: ylimits(0.4, 0.6)"
  print "Limits changed to 1.2<x<1.8, 0.4<y<0.6."
  if quitnow(): return
  unzoom()

  limits()
  print "Test 8:  Commands: limits"
  print "Limits back to extreme values (1,0) to (2,1)."
  if quitnow(): return
  unzoom()

  fma()
  x = 10*pi*arange(200, typecode = Float)/199.0
  plg(sin(x), x)
  print "Test 9:  Commands: fma(); plg(sin(x), x)"
  print "where x = 10*pi*arange(200, typecode = Float)/199.0"
  print "Five cycles of a sine wave on a new frame."
  print "Before you continue, try clicking with the mouse buttons:"
  print "Left button zooms in, right button zooms out, middle no zoom"
  print "In each case, the point where you press the mouse button will"
  print "be translated to the point where you release the mouse button."
  print "To zoom one axis only, click over the tick marks on that axis."
  sts = quitnow()
  unzoom()
  if (sts): return

  pledit(marks=0, width=6, type="dash")
  print "Test 10  Commands: pledit(marks=0, width=6, type=\"dash\")"
  print "Marker A on sine curve disappears, curve becomes bold dashed."
  if quitnow(): return
  unzoom()

  fma()
  x = 2*pi*arange(200, typecode = Float)/199.0
  for i in range(1,7):
    r = 0.5*i - (5-0.5*i)*cos(x)
    s = "curve [" + `i` + "]"
    plg( r*sin(x), r*cos(x), marks=0, color=-4-i, legend = s)
  print "Test 11:  Commands: plg(r*sin(x), r*cos(x), color=-4-i)"
  print "A set of nested cardioids in the primary and secondary colors."
  if quitnow(): return
  unzoom()

  pltitle ( "Cardioids" )
  plt ("Colored Nested Cardioids", -2.0, -3.0, orient = 1, opaque = 1, \
  tosys = 1, legend = "Some text")
  plq()
  print "Test 12:  Commands: pltitle(\"Cardioids\"); \
  plt(\"Colored nested cardioids\",-2.,-3.,orient=1,opaque=1,tosys=1); plq()"
  print "Adds the title above the upper tick marks."
  print "Also prints legends for the six curves at the terminal (plq)."
  if quitnow(): return
  unzoom()

  print "Test 13:  Commands: pledit(color=\"fg\", type=0, marker=i)"
  for i in range(1,6): pledit(i, color="fg", type=0, marker=i)
  pledit(i, color="fg", type=0, marker='A')
  print "Changes the colors to foreground, types to no lines."
  print "Markers are point, plus, asterisk, O, X, A."
  if quitnow(): return
  unzoom()

  print "Test 14:  Commands: pledit(marks=0, type=i)"
  for i in range(1,6): pledit(i, marks=0, type=i)
  pledit(i, color="fg", type=1, width=4)
  print "Changes line types to solid, dash, dot, dashdot, dashdotdot."
  print "Outermost cardioid becomes a thick, solid line."
  if quitnow(): return
  unzoom()

  fma()
  limits()
  x = a3(-1, 1, 26)
  y = transpose (x)
  z = x+1j*y
  z = 5.*z/(5.+z*z)
  xx = z.real
  yy = z.imaginary
  print "Test 15:  Commands: plm(y, x)"
  print "Quadrilateral mesh -- round with bites out of its sides."
  plm (yy, xx)
  if quitnow(): return
  fma()
  unzoom()

  print "Test 16:  Commands: plmesh ( y, x ); plv(v, u, y, x)"
  plmesh(yy, xx)
  plv(x+.5, y-.5)
  print "Velocity vectors.  Try zooming and panning with mouse."
  if quitnow(): return
  fma()
  unzoom()

  print "Test 17:  Commands: plfc(.); plc(.); plm(y,x, boundary=1, type=2)"
  ireg = ones (xx.shape, Int)
  ireg [0, :] = 0
  ireg [:, 0] = 0
  plfc(mag(x+.5,y-.5),yy,xx,ireg,contours=8)
  plc (mag(x+.5,y-.5), marks=1)
  plm (boundary=1, type=2)
# print "Contours A-H, with mesh boundary dashed."
  print "Contours I-P, with mesh boundary dashed."
  if quitnow(): return
  fma()
  unzoom()

  print "Test 18:  Commands: plf(z);  plfc(.); plc(z,.)"
  z = mag(x+.5,y-.5)
  plf(z)
  fma()
  unzoom()
  if quitnow(): return
  plfc(z,yy,xx,ireg,contours=array([0.5, 1.0, 1.5]))
  plc(z, marks=0, type=2, color="bg", levs=[0.5, 1.0, 1.5])
  print "Filled mesh (color only) with three dashed contours overlayed."
  if quitnow(): return
  fma()
  unzoom()

  print "Test 19:  Commands: palette( \"<various>.gp\")"
  print "After each new palette is installed, hit <RETURN> for the next,"
  print "or q<RETURN> to begin test 20.  There are 6 palettes altogether."
  z = mag(x+.5,y-.5)
  plf(z)
  plc(z, marks=0, type=2, color="bg", levs=[0.5, 1.0, 1.5])
  if quitnow(): return
  fma ()
  plfc(z,yy,xx,ireg,contours=array([0.5, 1.0, 1.5]))
  plc(z, marks=0, type=2, color="bg", levs=[0.5, 1.0, 1.5])
  pal = ["heat.gp", "stern.gp", "rainbow.gp", "gray.gp", "yarg.gp", "earth.gp"]
  for i in range(6):
    palette (pal[i])
    print "Palette name: ", pal[i]
    if quitnow(): return

  palette("earth.gp")
  fma()
  unzoom()

  print "Test 20:  Commands: window, style=\"<various>.gs\""
  print "After each new style is installed, hit <RETURN> for the next,"
  print "or q<RETURN> to begin test 21.  There are 5 styles altogether."
  pal= ["vg.gs", "boxed.gs", "vgbox.gs", "nobox.gs", "work.gs"]
  for i in range(5):
    window (style=pal[i])
    plc (mag(x+.5,y-.5), marks=1)
    plm (boundary=1, type=2)
    print "Style name: ", pal[i]
    if quitnow(): return

  window(style="work.gs")
  fma()
  unzoom()

  print "Test 21:  Commands: pli(image)"
  x = a3 (-6,6,200)
  y = transpose (x)
  r = mag(y,x)
  theta = arctan2 (y, x)
  funky = cos(r)**2 * cos(3*theta)
  pli(funky)
  print "Cell array image (color only).  Three cycles in theta, r."
  if quitnow(): return
  fma()
  unzoom()

  print "Test 22:  Commands: pldj(x0, y0, x1, y1)"
  theta = a2(0, 2*pi, 18)
  x = cos(theta)
  y = sin(theta)
  pldj(x, y, transpose (x), transpose (y))
  pltitle("Seventeen Pointed Stars")
  limits(square = 1)
  print "All 17 pointed stars."
  if quitnow(): return
  limits(square = 0)
  fma()
  unzoom()

  print "Test 23:  Commands: plfp (z, y, x, n)"

# .. This duplicates the test in yorick
# .. I do not know of a way to apply a "table lookup" in python;
# .. Hence, the temporary array.
#
# .. n = [ 5, 10, 15 ]
# .. list = [ 1, 1, 3, 3, 2, 1, 3 ]
# .. n(list) = [ 5, 5, 15, 15, 10, 5, 15 ] 

  n =  [3,4,5, 4, 5, 6, 5, 6, 7]
  n2 = [0,3,7,12,16,21,27,32,38]
  x0 = array ( [-2.,0.,2.]*3 )
  y0 = array ( [-2.]*3 + [0.]*3 + [2.]*3 )
  j = 1
  list = []
  for i in n:
     list = list + [j]*i
     j = j + 1
  lenList = len(list)

  nlist = arange(1,lenList+1) 
  n2list = arange(1,lenList+1) 
  phase = zeros ( (lenList), Float ) 
  theta = zeros ( (lenList), Float ) 
  x0List = zeros ( (lenList), Float ) 
  y0List = zeros ( (lenList), Float ) 

  for i in range(lenList):
     nlist[i] = n[list[i]-1]
     n2list[i] = n2[list[i]-1]
     x0List[i] = x0[list[i]-1]
     y0List[i] = y0[list[i]-1]

  phase = arange(1,lenList+1) - n2list
  theta = 2. * pi * phase / nlist
  x = cos(theta) + x0List
  y = sin(theta) + y0List

  plfp ( x0-y0, y, x, n, cmin=-4.5, cmax=4.5 )
  pltitle ( "Three rows of three polygons" )
  limits()
  print "Three rows of three polygons."
  if quitnow(): return
  fma()
  unzoom()

  print "Test 24:  Commands: plot histogram plh(...)"
  plh ( array([2.1, 7.3, 35.1]), labels=['Amsterdam', 'Bay Area', 'Tokyo'], 
        color=["magenta","cyan","yellow"] )
  if quitnow(): return
  fma()
  unzoom()

  print "Test 25:  Lissajous animation"
  print "First run without animation mode"
  print "Second run with animation mode"
  print "Press RETURN to continue"
  if quitnow(): return
  lissajous(0)
  print "Press RETURN to continue"
  if quitnow(): return
  lissajous(1)
  if quitnow(): return
  unzoom()

def quitnow(prompt=""):
  s = raw_input(prompt)
  if s == 'q': return 1
  return 0

def lissajous(animation):

#  DOCUMENT lissajous
#     runs the Yorick equivalent of an old graphics performance test
#     used to compare PLAN, ALMA, and Basis with LTSS TMDS graphics.
#   SEE ALSO: testg, grtest
#
#  Two figures with (x,y)= (cx,cy) + size*(cos(na*t), sin(nb*t+phase))
#  -- the centers describe semi-circular arcs of radius rc. 

   import os
   t= 2*pi*arange(400, typecode = Float)/399.0
   na1= 1
   nb1= 5
   na2= 2
   nb2= 7
   rc1= 40.
   rc2= 160.
   size= 40.
   phase= theta= 0.

   n = 200;   #/* number of frames in animation */
   dtheta= pi/(n-1)
   dphase= 2*pi/(n-1)

   if animation == 1:
      animate(1)
   else:
      animate(0)

   fma()

   t0 = os.times()

   for i in range(n):
      cost= cos(theta)
      sint= sin(theta)
      x= rc1*cost+size*cos(na1*t)
      y= rc1*sint+size*sin(nb1*t+phase);
      plg( y, x)
      x= rc2*cost+size*cos(na2*t)
      y= rc2*sint+size*sin(nb2*t+phase);
      plg( y, x)
      fma()
      theta= theta + dtheta;
      phase= phase + dphase;
  
   t1 = os.times()
   print "Timing summary for Lissajous"
   print "Wall clock:", t1[4] - t0[4]
   print "User cpu:", t1[0] - t0[0]
   print "Sys cpu:", t1[1] - t0[1]

   if animation == 1:
      #/* turn off animation and pop up final frame again */
      animate(0)
      x= -rc1+size*cos(na1*t)
      y= size*sin(nb1*t);
      plg(y, x)
      x= -rc2+size*cos(na2*t)
      y= size*sin(nb2*t);
      plg(y, x)

# (nr X nc) array, each row an integer in the sequence [0, 1, ... ]
def a1(nr,nc):
   return reshape (array(concat1(nr,nc), Float), (nr,nc))

# (n-1) square array, with each row == spanz(lb,ub,n)
def a2(lb,ub,n):
   return reshape (array((n-1)*spanz(lb,ub,n), Float), (n-1,n-1))
  
# (n) square array, with each row == span(lb,ub,n)
def a3(lb,ub,n):
   return reshape (array(n*span(lb,ub,n), Float), (n,n))
  
# Half-hearted attempt at span(), which returns N equally-spaced
# values in the sequence [lb, ..., ub]
def span(lb,ub,n):
   if n < 2: raise ValueError, '3rd arg must be at least 2'
   b = lb
   a = (ub - lb)/(n - 1.0)
   #return a*arange(n) + b # if you want an array
   return map(lambda x,A=a,B=b: A*x + B, range(n)) # if you want a list

# Half-hearted attempt at span()(zcen), which returns N-1 "zone-centered"
# values in sequence (lb, ..., ub)
def spanz(lb,ub,n):
   if n < 3: raise ValueError, '3rd arg must be at least 3'
   c = 0.5*(ub - lb)/(n - 1.0)
   b = lb + c
   a = (ub - c - b)/(n - 2.0)
   return map(lambda x,A=a,B=b: A*x + B, range(n-1))

# magnitude or distance function
def mag(*args):
   r = 0
   for i in range(len(args)):
      r = r + args[i]*args[i]
   return sqrt(r)

def run():
   grtest()

def demo():
   grtest()

if __name__ == '__main__': 
   run()



#!/usr/bin/env python
#
# Created by: Pearu Peterson, October 2003
#
__usage__ = """
First ensure that scipy_core modules are installed.

Build xxx:
  python setup_xxx.py build
Run tests locally:
  python tests/test_foo.py [-l<int>] [-v<int>]

Run tests if scipy is installed:
  python -c 'import scipy;scipy.xxx.test(level=<int>,verbosity=<int>)'
"""

import sys
from scipy_test.testing import *

set_package_path()
from xxx.foo import *
del sys.path[0]

class test_foo_bar(ScipyTestCase):

    def check_simple(self, level=1):
        assert xxx_foo_bar()=='Hello from xxx_foo_bar'

class test_foo_gun(ScipyTestCase):

    def check_simple(self, level=1):
        assert foo_gun()=='Hello from foo_gun'

if __name__ == "__main__":
    ScipyTest('xxx.foo').run()



#!/usr/bin/env python

import os
from scipy_distutils.core import Extension
from scipy_distutils.misc_util import get_path, dot_join, default_config_dict

def configuration(parent_package='',parent_path=None):
    package = 'io'
    local_path = get_path(__name__,parent_path)
    config = default_config_dict(package,parent_package)

    sources = ['numpyiomodule.c']
    sources = [os.path.join(local_path,x) for x in sources]
    ext = Extension(dot_join(parent_package,package,'numpyio'),sources)
    config['ext_modules'].append(ext)
    return config

if __name__ == '__main__':    
    from scipy_distutils.core import setup
    setup(**configuration())



#
# signal - Signal Processing Tools
#

from info_signal import __doc__

import sigtools
from waveforms import *
from bsplines import *
from filter_design import *
from ltisys import *
from signaltools import *




#this program corresponds to special.py
import os
#import fpformat
import unittest
import sys
from unittest import TestCase
import scipy_base.limits as limits
from scipy_test.testing import *
set_package_path()
import scipy.signal as signal
del sys.path[0]

from Numeric import array

class test_convolve(TestCase):
    def check_basic(self):
        a = [3,4,5,6,5,4]
        b = [1,2,3]
        c = signal.convolve(a,b)
        assert_array_equal(c,array([3,10,22,28,32,32,23,12]))

class test_medfilt(TestCase):
    def check_basic(self):
        f = [[3,4,5],[2,3,4],[1,2,5]]
        d = signal.medfilt(f)
        assert_array_equal(d, [[0,3,0],[2,3,3],[0,2,0]])

class test_wiener(TestCase):
    def check_basic(self):
        g = array([[5,6,4,3],[3,5,6,2],[2,3,5,6],[1,6,9,7]],'d')
        correct = array([[2.16374269,3.2222222222, 2.8888888889, 1.6666666667],[2.666666667, 4.33333333333, 4.44444444444, 2.8888888888],[2.222222222, 4.4444444444, 5.4444444444, 4.801066874837],[1.33333333333, 3.92735042735, 6.0712560386, 5.0404040404]])
        h = signal.wiener(g)
        assert_array_almost_equal(h,correct,decimal=6)

if __name__ == "__main__":
    ScipyTest(signal).run()









#!/usr/bin/env python
# Created by Pearu Peterson, September 2002
""" Test functions for fftpack.helper module
"""
__usage__ = """
Build fftpack:
  python setup_fftpack.py build
Run tests if scipy is installed:
  python -c 'import scipy;scipy.fftpack.test(<level>)'
Run tests if fftpack is not installed:
  python tests/test_helper.py [<level>]
"""

import sys
from scipy_test.testing import *
set_package_path()
from fftpack import fftshift,ifftshift,fftfreq,rfftfreq
del sys.path[0]


import Numeric
from Numeric import pi

def random(size):
    return rand(*size)

import unittest

class test_fftshift(ScipyTestCase):

    def check_definition(self):
        x = [0,1,2,3,4,-4,-3,-2,-1]
        y = [-4,-3,-2,-1,0,1,2,3,4]
        assert_array_almost_equal(fftshift(x),y)
        assert_array_almost_equal(ifftshift(y),x)
        x = [0,1,2,3,4,-5,-4,-3,-2,-1]
        y = [-5,-4,-3,-2,-1,0,1,2,3,4]
        assert_array_almost_equal(fftshift(x),y)
        assert_array_almost_equal(ifftshift(y),x)

    def check_inverse(self):
        for n in [1,4,9,100,211]:
            x = random((n,))
            assert_array_almost_equal(ifftshift(fftshift(x)),x)

class test_fftfreq(ScipyTestCase):

    def check_definition(self):
        x = [0,1,2,3,4,-4,-3,-2,-1]
        assert_array_almost_equal(9*fftfreq(9),x)
        assert_array_almost_equal(9*pi*fftfreq(9,pi),x)
        x = [0,1,2,3,4,-5,-4,-3,-2,-1]
        assert_array_almost_equal(10*fftfreq(10),x)
        assert_array_almost_equal(10*pi*fftfreq(10,pi),x)

class test_rfftfreq(ScipyTestCase):

    def check_definition(self):
        x = [0,1,1,2,2,3,3,4,4]
        assert_array_almost_equal(9*rfftfreq(9),x)
        assert_array_almost_equal(9*pi*rfftfreq(9,pi),x)
        x = [0,1,1,2,2,3,3,4,4,5]
        assert_array_almost_equal(10*rfftfreq(10),x)
        assert_array_almost_equal(10*pi*rfftfreq(10,pi),x)

if __name__ == "__main__":
    ScipyTest('fftpack.helper').run()



# Copyright (c) 1996, 1997, The Regents of the University of California.
# All rights reserved.  See Legal.htm for full text and disclaimer.

# The following more or less permanent defaults can be overruled
# by assignment, provided that this file is execfile'd rather than import'ed.
defgridx = "off"
defgridy = "off"
defthick = 1.0
defmark = " "
defstyle = "solid"
deflabels = 1
deflabel = None
defscale = "linlin"
defcolor = "fg"
deftop = ""
defbot = ""
defleft = ""
defright = ""
defvsc = .5
deflev = 8
defcgm = "yes"
defps = "no"

_null_object_list_ = [ [], [], [], [], [], [], [], []]

def _set_frame_defaults_ ( ) :
   global _titles_, _object_list_, _grid_type_, _axis_labels_, _axis_limits_, \
          _axis_scales_, _gridx_, _gridy_, _thick_, _mark_, _style_, \
          _labels_enabled_, _axis_scales_, _color_, _label_list_, _text_, \
          _text_pos_, _text_size_, _text_color_, _tosys_, _cmin_, \
          _cmax_, _vsc_, _lev_, _krange_, _lrange_, _kstyle_, _lstyle_, \
          _region_, _default_label_, _color_bar_
   _region_ = "all"
   _lev_ = deflev
   _krange_ = None
   _lrange_ = None
   _kstyle_ = "solid"
   _lstyle_ = "solid"
   _vsc_ = defvsc
   _titles_ = [defbot, deftop,
            defleft, defright]  # bottom, top, left, right titles
   _object_list_ = [ [], [], [], [], [], [], [], []] # (ensure copy!!)
      # list of objects to plot (one for each possible device)
   _label_list_ = []
   _text_ = [" "]
   _text_pos_ = [ [0., 0.]]
   _text_size_ = [0]
   _text_color_ = ["bg"]
   _tosys_ = [1]
   _gridx_ = defgridx
   _gridy_ = defgridy
   _grid_type_ = "axes"         # grid type for current graph
   _axis_labels_ = ["X axis",
                    "Y axis",
                    "YR axis"]  # axis labels
   _axis_limits_ = [["d","d"],
                    ["d","d"],
                    ["d","d"]]  # axis limits; "d" implies default.
   _axis_scales_ = ["lin",
                    "lin",
                    "lin"]      # axis scales
   _thick_ = defthick
   _mark_ = defmark
   _style_ = defstyle
   _labels_enabled_ = deflabels
   _default_label_ = " "
   _axis_scales_ = defscale
   _color_ = defcolor
   _cmin_ = _cmax_ = 0.0
   _color_bar_ = 0

def _set_initial_defaults_ ( ) :
   # Set defaults
   global _ezcshow_, _ezcreset_, _window_, _cgm_, _ps_, _displays_, \
          _ps_file_, _cgm_plotter_, _ps_plotter_, _win_plotters_, \
          _current_graph_, _cgm_file_, _graphics_, _win_on_, _zt_, \
          _rt_, _ireg_, _pvar_, _cindex_, _ut_, _vt_
   _ezcshow_ = "true"           # no plot until nf
   _ezcreset_ = "true"          # return to initial values after nf
   _window_ = "no"              # no window
   _cgm_ = defcgm               # create cgm file
   _ps_ = defps                 # create postscript file
   _displays_ = [" ", " ", " ",
                 " ", " ", " ",
                 " ", " "]      # hosts where display occurs
   _graphics_ = ["", "", "",
                 "", "", "",
                 "", ""]        # type of graphics to use
   _ps_file_ = "Aa00.ps"        # name of postscipt file for display
   _cgm_file_ = "Aa00.cgm"      # name of cgm file for display
   _cgm_plotter_ = -1           # plotter which writes to cgm file
                                # (subscript into _win_plotters_)
   _ps_plotter_ = -1            # plotter which writes to postscript file
                                # (subscript into _win_plotters_)
   _win_plotters_ = [None,
                     None,
                     None,
                     None,
                     None,
                     None,
                     None,
                     None]      # plotters which plot to windows (max 8)
                                # _cgm_plotter_ and _ps_plotter_ are
                                # on this list, if they exist.
   _win_on_ = [0, 0, 0, 0,
               0, 0, 0, 0]      # windows actually activated
   _current_graph_ = [None,
                      None,
                      None,
                      None,
                      None,
                      None,
                      None,
                      None]      # current graph2d object
   _set_frame_defaults_ ( )
   _zt_ = None
   _rt_ = None
   _ireg_ = None
   _pvar_ = None
   _cindex_ = None
   _ut_ = None
   _vt_ = None

_set_initial_defaults_ ( )

GraphicsError = "GraphicsError"
import os
try:
   graphics = os.environ["PYGRAPH"]
except KeyError:
   graphics = "Gist"

from gist import * # to get mfit function


if graphics [0:3] == "Nar" :
   import NarPlotter
   GistPlotter = None
   Graphics = NarPlotter
   print "Opening Narcisse graphics."
elif graphics == "Gist" :
   import GistPlotter
   NarPlotter = None
   Graphics = GistPlotter
   print "Opening Gist graphics."
else :
   raise GraphicsError , \
      graphics + " is an unknown graphics package. Check PYGRAPH " + \
         "environment variable."

from region import *
from quadmesh import *
from curve import *
from graph2d import *
from cellarray import *

import shapetest
from Numeric import *
from scipy_base.fastumath import *

_ezdict_ = {'t': 'true' , 'T': 'true', 'y': 'true', 'Y': 'true',
            'f': 'false', 'F': 'false', 'n': 'false', 'N': 'false'}

def ezcshow (val) :
   global _ezcshow_, _ezdict_
   EzcShowErr = "EzcShowErr"
   if not _ezdict_.has_key (val [0][0]) :
      raise EzcShowErr , \
         "Illegal argument! must be 'true' or 'false'."
   _ezcshow_ = _ezdict_ [val [0][0]]

def ezcreset (val) :
   global _ezcreset_, _ezdict_
   EzcResetErr = "EzcResetErr"
   if not _ezdict_.has_key (val [0][0]) :
      raise EzcResetErr , \
         "Illegal argument! must be 'true' or 'false'."
   _ezcreset_ = _ezdict_ [val [0][0]]

from gistC import bytscl

def ezcfmc (val) :
   """returns an array of indices into the current palette."""
   return bytscl (val)

_WinSpecError_ = "WinSpecError"

def _get_free_win_number_ ( ) :
   # Gets the lowest number not associated with an open device, else -1.
   global _win_on_
   try :
      return _win_on_.index (0)
   except:
      return -1

def _get_first_open_win_number () :
   # Gets the number of the lowest numbered open window, if any, else -1..
   global _win_on_, _cgm_plotter_, _ps_plotter_
   for i in range (8) :
      if _win_on_ [i] != 0 and i != _cgm_plotter_ and i != _ps_plotter_:
         return i
   return -1

def win (val , *n, **keywords) :
   """win (str [, n] [, display = <string1>] [, graphics = <string2>])
         str = "on" : reserves a graphics window for display. n, if
            present, must be between 0 and 7 and specifies which window.
            if n is absent, it will default to the lowest-numbered window
            available. display, if present, specifies via <string1>
            where the window will be displayed, e. g., "icf.llnl.gov:0.0".
            The default is $DISPLAY. graphics, if present, specifies
            via <string2> which type of graphics is desired, "Nar"
            for Narcisse, and "Gist" for Gist being the currently
            allowed values. The default is $PYGRAPH if set, otherwise
            "Gist".
         str = "close" : closes a graphics window; wndow n if
            specified, otherwise window 0.
   """
   global _win_on_, _displays_, _ps_plotter_, _cgm_plotter_, _graphics_, \
      _win_plotters_, _object_list_
   if len (n) > 0 :
      _window_number_ = n [0]
   else :
      if keywords.has_key ("window") :
         _window_number_ = keywords ["window"]
      else :
         _window_number_ = "all"
   if val == "on" or val == "ON" or val == "On" \
      or val == "open" or val == "OPEN" or val == "Open" :
      _window_number_ = _check_win_ (_window_number_)
      if _window_number_ == []:
         _window_number_ = [_get_free_win_number_ ( )]
         if _window_number_ [0] < 0 :
            raise _WinSpecError_, "win ('open'): all windows are in use."
      for i in _window_number_:
         if i < 0 or i > 7 :
            raise _WinSpecError_, "Window number (" + `i` + \
               ") is not between 0 and 7."
         _win_on_ [i] = 1
         if keywords.has_key ("display") :
            _displays_ [i] = keywords ["display"]
         if keywords.has_key ("graphics") :
            _graphics_ [i] = keywords ["graphics"]
   if val == "off" or val == "OFF" or val == "Off" or \
      val == "close" or val == "CLOSE" or val == "Close" :
      _window_number_ = _check_win_ (_window_number_, minmax = 1)
      if _window_number_ == []:
         _window_number_ = [0]
      for i in _window_number_:
         if i < 0 or i > 7 :
            raise _WinSpecError_, "win ('close'): window number (" + `i` + \
               ") is not between 0 and 7."
         _displays_ [i] = " "
         _graphics_ [i] = ""
         _win_on_ [i] = 0
         _object_list_ [i] = []
         if _win_plotters_ [i] is not None :
            _win_plotters_ [i].close ()
            _win_plotters_ [i] = None
         if i == _ps_plotter_ :
            _ps_plotter_ = -1
         elif i == _cgm_plotter_ :
            _cgm_plotter_ = -1

tv = win

_CgmError_ = "CgmError"

def new_plot_file ( filename ) :
   """new_plot_file ( ) will eventually return the name of the next
   available file in the sequence "Aa00.ps/cgm", Ab00.ps/cgm", etc.
   """
   if not os.path.isfile ("./" + filename) :
      return filename
   n = string.find (string.letters, filename [1])
   if n == -1 or n == len (string.letters) - 1 :
      raise _PsError_ , "No more ps filenames available."
   letters = string.letters [n + 1:]
   OK = 0
   for l in string.letters :
      tmpfile = filename[0:1] + l + filename[2:]
      if not os.path.isfile ("./" + tmpfile) :
         OK = 1
         break
   if OK == 1 :
      return tmpfile
   else :
      raise _PsError_ , "No more ps or cgm filenames available."

def _min_ ( x ) :
   if is_scalar (x) :
      return x
   else :
      return min (ravel (x))

def _max_ ( x ) :
   if is_scalar (x) :
      return x
   else :
      return max (ravel (x))

def _get_objects_ () :
   global _object_list_, _null_object_list_
   _this_object_list_ = []
   if _object_list_ != _null_object_list_ :
      for i in range (8):
         if _object_list_ [i] != [] :
            _this_object_list_.append (i)
   return _this_object_list_

def _get_open_win_list ():
   global _win_on_
   openl = []
   for i in range (len (_win_on_)) :
      if _win_on_ [i]:
         openl.append (i)
   return openl


_BadWindow = "BadWindow"

def _check_win_ (window, minmax = 0) :
   """_check_win_ (window) checks the integrity of the window specification,
   and if OK, returns a list of windows. If minmax = 0, treats "min" and "max"
   as errors, otherwise allows them."""
   global _win_on_
   if window == "all" :
      return _get_open_win_list ()
   elif minmax == 1 and window == "min" :
      for i in range (len (_win_on_)) :
         if _win_on_ [i]:
            return [i]
   elif minmax == 1 and window == "max" :
      for i in range (len (_win_on_)) :
         if _win_on_ [7 - i]:
            return [7 - i]
   elif type (window) == IntType and 0 <= window <= 7:
      return [window]
   elif type (window) == ListType:
      err = 0
      for i in range (len (window)):
         if type (window [i]) == IntType and 0 <= window [i] <= 7:
            continue
         else:
            err = 1
            break
      if err == 0:
         return window
   raise _BadWindow, "Incorrect window specification: " + `window` + "."

def _set_axis_limits_ (window = "all") :
   """_set_axis_limits_ ( ) makes sure legal values are used for the defaults."""
   global _axis_limits_, _object_list_, _null_object_list_
   ## Someday this will be improved to have a list of object lists, one for each device.
   _object_numbers_ = _check_win_ (window, minmax = 1)
   if _object_numbers_ == []:
      _axis_limits_ = [ [0., 0.], [0., 0.], [0., 0.]]
      return
   min_ = max_ = None
   for j in range (len (_object_numbers_)):
      if _object_list_ [j] == [] :
         continue
      for i in range (3) :
         if _axis_limits_ [i] == ["d", "d"] :
            _axis_limits_ [i] = [0., 0.]
            continue
         if _axis_limits_ [i] [0] == "d" :
            if i == 0 :
               for k in range (len (_object_list_ [j])) :
                  new_min = _min_ (_object_list_ [j] [k].x)
                  if min_ is None or new_min < min_ :
                     min_ = new_min
            elif i == 1 :
               for k in range (len (_object_list_ [j])) :
                  new_min = _min_ (_object_list_ [j] [k].y)
                  if min_ is None or new_min < min_ :
                     min_ = new_min
            elif i == 2 :
               min_ = 0.
            _axis_limits_ [i] [0] = min_
         if _axis_limits_ [i] [1] == "d" :
            if i == 0 :
               for k in range (len (_object_list_ [j])) :
                  new_max = _max_ (_object_list_ [j] [k].x)
                  if max_ is None or new_max > max_ :
                     max_ = new_max
            elif i == 1 :
               for k in range (len (_object_list_ [j])) :
                  new_max = _max_ (_object_list_ [j] [k].y)
                  if max_ is None or new_max > max_ :
                     max_ = new_max
            elif i == 2 :
               max_ = 0.
            _axis_limits_ [i] [1] = max_

_CgmError_ = "CgmError"

def _get_win_ (window, error):
   """_get_win_ (window) returns the number of a window which has a
   non-empty _object_list_. Has an exception if there is none--nothing to plot."""
   global _object_list_
   win_num = -1
   if window == "min":
      # Find the earliest window with a nonempty list (if any)
      for i in range (8):
         if _object_list_ [i] != []:
            win_num = i
            break

   elif window == "max":
      # Find the last window with a nonempty list (if any)
      for i in range (8):
         if _object_list_ [7 - i] != []:
            win_num = 7 - i
            break

   elif 0 <= window <= 7:
      # Check for a nonempty object list:
      if _object_list_ [window] != [] :
         win_num = window

   else:
      # Find the earliest window with a nonempty list (if any)
      for i in range (8):
         if _object_list_ [i] != []:
            win_num = i
            break
      
   if win_num == -1:
      raise error, "There seems to be nothing there!"
   return win_num
   
def cgm (val, *n, **kw) :
   global _cgm_, _cgm_plotter_, _current_graph_, _cgm_file_, _win_on_, \
          _win_plotters, _cmin_, _cmax_, _titles_, _axis_limits_, \
          _axis_labels_, _axis_scales_, _text_, _text_pos_, \
          _text_size_, _text_color_, _tosys_, _grid_type_, _object_list_
   if kw.has_key ("new_frame") :
      new_frame = kw ["new_frame"]
   elif _ezcshow_ == "false" :
      new_frame = "no"
   else :
      new_frame = "yes"
   if val == "on" or val == "ON" or val == "On" \
      or val == "open" or val == "OPEN" or val == "Open" :
      _cgm_ = "yes"
      if len (n) == 0 :
         if 7 >= _cgm_plotter_ >= 0 :
            if _win_on_ [_cgm_plotter_] :
               return # a window is already assigned to cgm and is open
            else :
               _win_on_ [_cgm_plotter_] = 1
               return # a window is already assigned to cgm
         else :
            _window_number_ = _get_free_win_number_ ()
            if _window_number_ < 0 :
               raise _CgmError_, "No window available for cgm use."
      else :
         _window_number_ = n [0]
         if _window_number_ < 0 or _window_number_ > 7 :
            raise _CgmError_, "Window number (" + `_window_number_` + \
               ") is not between 0 and 7."
         if 0 <= _cgm_plotter_ <= 7 :
            if _cgm_plotter_ == _window_number_ :
               return # this window is already assigned to cgm
            else :
               # another cgm window was assigned. unassign it.
               _win_on_ [_cgm_plotter_] = 0
               if _win_plotters_ [_cgm_plotter_] is not None :
                  _win_plotters_ [_cgm_plotter_].close ()
                  _win_plotters_ [_cgm_plotter_] = None
      _cgm_plotter_ = _window_number_
      _win_on_ [_cgm_plotter_] = 1
      return
   if val == "close" or val == "CLOSE" or val == "Close" :
      _cgm_ = "no"
      if 0 <= _cgm_plotter_ <= 7 :
         print "Closing cgm file " + _cgm_file_ + "."
         _win_on_ [_cgm_plotter_] = 0
         if _win_plotters_ [_cgm_plotter_] is not None :
            _win_plotters_ [_cgm_plotter_].close ()
            _win_plotters_ [_cgm_plotter_] = None
         _cgm_plotter_ = -1
      return
   if val == "off" or val == "OFF" or val == "Off" :
      # Turn off transmissions to window but don't close
      _cgm_ = "no"
      return
   if val == "send" or val == "SEND" or val == "Send" or val == "plot" :
      # Figure out which object list is to be sent
      if _cgm_plotter_ < 0 or _object_list_ [_cgm_plotter_] == 0 and \
         not kw.has_key ("window"):
         kw ["window"] = "min"
      if kw.has_key ("window") :
         # User has specified a window whose object list is to be plotted
         # or else the cgfm window doesn't have a plotting list of its own
         # -- guaranteed to return a valid window number; exception otherwise.
         win_no = _get_win_ (kw ["window"], _CgmError_)
      # time to send a graph to a cgm file
      if _cgm_plotter_ < 0 :
         _cgm_plotter_ = _get_free_win_number_ ()
         if _cgm_plotter_ < 0 :
            raise _CgmError_, "No window available for cgm use."
         _win_on_ [_cgm_plotter_] = 1
      if _win_plotters_ [_cgm_plotter_] is None :
         _cgm_file_ = new_plot_file (_cgm_file_)
         print "Opening plot file " + _cgm_file_ + "."
         _win_plotters_ [_cgm_plotter_] = \
            GistPlotter.Plotter ( "none" , n = _cgm_plotter_, hcp = _cgm_file_)
      _win_plotters_ [_cgm_plotter_].set_bytscl (_cmin_, _cmax_)
      _set_axis_limits_ ( )
      if _cgm_plotter_ != win_no :
         _object_list_ [_cgm_plotter_] = _object_list_ [win_no]
      if _current_graph_ [_cgm_plotter_] is None :
         _current_graph_ [_cgm_plotter_] = Graph2d (_object_list_ [win_no], \
            plotter = _win_plotters_ [_cgm_plotter_], \
            titles = _titles_, axis_limits = _axis_limits_, axis_labels = \
            _axis_labels_, axis_scales = _axis_scales_,  \
            text = _text_, text_pos = _text_pos_, text_size = \
            _text_size_, text_color = _text_color_, tosys = _tosys_, \
            grid_type = _grid_type_)
      elif val == "plot" :
          _current_graph_ [_cgm_plotter_].set (plotter = _win_plotters_ [_cgm_plotter_],
              axis_limits = _axis_limits_)
      else :
         _current_graph_ [_cgm_plotter_].new (_object_list_ [win_no], \
            plotter = _win_plotters_ [_cgm_plotter_], \
            text = _text_, text_pos = _text_pos_, text_size = \
            _text_size_, text_color = _text_color_, tosys = _tosys_, \
            titles = _titles_, axis_limits = _axis_limits_, axis_labels = \
            _axis_labels_, axis_scales = _axis_scales_, grid_type = _grid_type_)
      if new_frame == "no" :
         _current_graph_ [_cgm_plotter_].plot ( )
         _win_plotters_ [_cgm_plotter_].plot_text ( )
      if val != "plot" : # Got to turn off the file.
         _cgm_ = "no"

import string

_PsError_ = "PsError"

def ps (val, *n, **kw) :
   global _ps_, _ps_plotter_, _current_graph_, _ps_file_, _win_on_, \
          _win_plotters, _cmin_, _cmax_, _titles_, _axis_limits_, \
          _axis_labels_, _axis_scales_, _text_, _text_pos_, \
          _text_size_, _text_color_, _tosys_, _grid_type_, _object_list_
   if kw.has_key ("new_frame") :
      new_frame = kw ["new_frame"]
   elif _ezcshow_ == "false" :
      new_frame = "no"
   if val == "on" or val == "ON" or val == "On" \
      or val == "open" or val == "OPEN" or val == "Open" :
      _ps_ = "yes"
      if len (n) == 0 :
         if 7 >= _ps_plotter_ >= 0 :
            if _win_on_ [_ps_plotter_] :
               return # a window is already assigned to ps
            else :
               _win_on_ [_ps_plotter_] = 1
               return
         else :
            _window_number_ = _get_free_win_number_ ()
            if _window_number_ < 0 :
               raise _PsError_, "No window available for ps use."
      else :
         _window_number_ = n [0]
         if _window_number_ < 0 or _window_number_ > 7 :
            raise _PsError_, "Window number (" + `_window_number_` + \
               ") is not between 0 and 7."
         if 0 <= _ps_plotter_ <= 7 :
            if _ps_plotter_ == _window_number_ :
               return # this window is already assigned to ps
            else :
               # another ps window was assigned. unassign it.
               _win_on_ [_ps_plotter_] = 0
               if _win_plotters_ [_ps_plotter_] is not None :
                  _win_plotters_ [_ps_plotter_].close ()
                  _win_plotters_ [_ps_plotter_] = None
      _ps_plotter_ = _window_number_
      _win_on_ [_ps_plotter_] = 1
      return
   if val == "off" or val == "OFF" or val == "Off" :
      _ps_ = "no"
      return
   if val == "close" or val == "CLOSE" or val == "Close" :
      _ps_ = "no"
      if 0 <= _ps_plotter_ <= 7 :
         _win_on_ [_ps_plotter_] = 0
         if _win_plotters_ [_ps_plotter_] is not None :
            _win_plotters_ [_ps_plotter_].close ()
            _win_plotters_ [_ps_plotter_] = None
         _ps_plotter_ = -1
         print "Closing plot file " + _ps_file_ + "."
      return
   if val == "send" or val == "SEND" or val == "Send" or val == "plot" :
      # Figure out which object list is to be sent
      if _ps_plotter_ < 0 or _object_list_ [_ps_plotter_] == 0 and \
         not kw.has_key ("window"):
         kw ["window"] = "min"
      if kw.has_key ("window") :
         # User has specified a window whose object list is to be plotted
         # or else the cgfm window doesn't have a plotting list of its own
         # -- guaranteed to return a valid window number; exception otherwise.
         win_no = _get_win_ (kw ["window"], _PsError_)
      # time to send a graph to a ps file
      if _ps_plotter_ < 0 :
         _ps_plotter_ = _get_free_win_number_ ()
         if _ps_plotter_ < 0 :
            raise _PsError_, "No window available for ps use."
         _win_on_ [_ps_plotter_] = 1
      if _win_plotters_ [_ps_plotter_] is None :
         _ps_file_ = new_plot_file (_ps_file_)
         print "Opening plot file " + _ps_file_ + "."
         _win_plotters_ [_ps_plotter_] = \
            GistPlotter.Plotter ( "none" , n = _ps_plotter_, hcp = _ps_file_)
      _win_plotters_ [_ps_plotter_].set_bytscl (_cmin_, _cmax_)
      _set_axis_limits_ ( )
      if _ps_plotter_ != win_no :
         _object_list_ [_ps_plotter_] = _object_list_ [win_no]
      if _current_graph_ [_ps_plotter_] is None :
         _current_graph_ [_ps_plotter_] = Graph2d (_object_list_ [win_no], \
            plotter = _win_plotters_ [_ps_plotter_], \
            titles = _titles_, axis_limits = _axis_limits_, axis_labels = \
            _axis_labels_, axis_scales = _axis_scales_,  \
            text = _text_, text_pos = _text_pos_, text_size = \
            _text_size_, text_color = _text_color_, tosys = _tosys_, \
            grid_type = _grid_type_)
      elif val == "plot" :
          _current_graph_ [_ps_plotter_].set (plotter = _win_plotters_ [_ps_plotter_],
              axis_limits = _axis_limits_)
      else :
         _current_graph_ [_ps_plotter_].new (_object_list_ [win_no], \
            plotter = _win_plotters_ [_ps_plotter_], \
            text = _text_, text_pos = _text_pos_, text_size = \
            _text_size_, text_color = _text_color_, tosys = _tosys_, \
            titles = _titles_, axis_limits = _axis_limits_, axis_labels = \
            _axis_labels_, axis_scales = _axis_scales_, grid_type = _grid_type_)
      if new_frame == "no" :
         _current_graph_ [_ps_plotter_].plot ( )
         _win_plotters_ [_ps_plotter_].plot_text ( )
      if val != "plot" : # Got to turn off the file.
         _ps_ = "no"


_NfError_ = "NfError"
_UndoError_ = "UndoError"

def undo (item = None, window = "min") :
   """undo (item, window) removes the item-th object from window's
   object list, if it can. item counts starting at 1."""
   global _object_list_
   win_no = _get_win_ (window, _UndoError_)
   lo = len (_object_list_ [win_no])
   if item is None :
      n = lo
   if n > lo or n < 1:
      raise _UndoError_, "There aren't " + `n` + " objects in the list."
   _object_list_ [win_no] [n - 1:n] = []
   if _object_list_ [win_no] != [] and _ezcshow_ == "true" :
      sf (win_no)
   
def nf ( new_frame = "yes" , window = "all" ) :
   global _win_plotters_, _current_graph_, _object_list_, _label_list_, \
          _cmin_, _cmax_, _cgm_, _ps_, _labels_enabled_, _color_bar_, \
          _win_on_, _cgm_plotter_, _ps_plotter_, _titles_, _axis_limits_, \
          _axis_labels_, _axis_scales_, _text_, _text_pos_, _text_size_, \
          _text_color_, _tosys_, _grid_type_, _object_list_, _graphics_, \
          _current_graph_, _labels_temporarily_enabled_
   enablen = zeros (8, Int)
   enable_cgm = 0
   enable_ps = 0
   if _object_list_ == _null_object_list_ :
      # Nothing to plot
      return
   if _cgm_ == "yes" and _cgm_plotter_ < 0 :
      # Open default cgm plotter
      print "Opening default cgm plotter."
      cgm ("on")
   if window == "cgm" :
      enable_cgm = 1
   elif window == "ps" :
      enable_ps = 1
   else :
      windows = _check_win_ (window, minmax = 1)
      for i in windows:
         enablen [i] = 1
   _set_axis_limits_ ( )
   for i in range (len (_win_plotters_)) :
      for j in range (len (_object_list_ [i])) :
         if _labels_temporarily_enabled_ and len (_label_list_) > j :
            _object_list_ [i] [j].set (label = _label_list_ [j])
            _object_list_ [i] [j].set (marks = 1)
         else :
            _object_list_ [i] [j].set (label = " ")
            _object_list_ [i] [j].set (marks = 0)
#           if _object_list_ [i] [j].line_type != "none" :
#              _object_list_ [i] [j].set (marks = 0)
      if i == _cgm_plotter_ and (enable_cgm or enablen [i]) and _cgm_ :
         cgm ("plot", new_frame = new_frame, window = _cgm_plotter_)
      elif i == _ps_plotter_ and (enable_ps or enablen [i]) and _ps_ :
         ps ("plot", new_frame = new_frame, window = _ps_plotter_)
      elif enablen [i] :
         if _current_graph_ [i] is None :
            _current_graph_ [i] = Graph2d (_object_list_ [i], titles = _titles_,
               text = _text_, text_pos = _text_pos_, text_size = \
               _text_size_, text_color = _text_color_, tosys = _tosys_, \
               axis_limits = _axis_limits_, axis_labels =_axis_labels_,
               axis_scales = _axis_scales_, grid_type = _grid_type_,
               color_bar = _color_bar_)
         else :
            _current_graph_ [i].new (_object_list_ [i], titles = _titles_,
               text = _text_, text_pos = _text_pos_, text_size = \
               _text_size_, text_color = _text_color_, tosys = _tosys_, \
               axis_limits = _axis_limits_, axis_labels =_axis_labels_,
               axis_scales = _axis_scales_, grid_type = _grid_type_,
               color_bar = _color_bar_)
         if _win_on_ [i] == 1 :
            if _win_plotters_ [i] is None :
               if _graphics_ [i] [0:3] == "Nar" :
                  gr = NarPlotter
               elif _graphics_ [i] == "Gist" :
                  gr = GistPlotter
               else :
                  gr = Graphics
               _win_plotters_ [i] = gr.Plotter (n = i, display = _displays_ [i])
            if new_frame == "no" :
               _win_plotters_ [i].set_bytscl (_cmin_, _cmax_)
               _current_graph_ [i].set (plotter = _win_plotters_ [i])
               _current_graph_ [i].plot ( )
               _win_plotters_ [i].plot_text ( )
         if new_frame == "yes" :
            _object_list_ [i] = []
            _label_list_ = []
            if _ezcreset_ == 'true' :
               _set_frame_defaults_ ( )

def sf ( window = "all") :
   nf ("no", window = window)

def display (vals) :
   global _displays_
   if shapetest.is_scalar (vals) :
      _displays_ = [vals]
   else :
      _displays_ = vals

def titles (* vals) :
   global _titles_
   if len (vals) == 0 :
      _titles_ = [defbot, deftop, defleft, defright]
   elif len (vals) == 1 :
      _titles_ = [defbot, vals [0], defleft, defright]
   elif len (vals) >= 2 :
      _titles_ [0] = vals [1]
      _titles_ [1] = vals [0]
      if len (vals) >= 3 :
         _titles_ [2] = vals [2]
         if len (vals) >= 4 :
            _titles_ [3] = vals [3]
         else :
            _titles_ [3] = defright
      else :
         _titles_ [2] = defleft
         _titles_ [3] = defright

def titleb (val) :
   global _titles_
   _titles_ [0] = val

def titlet (val) :
   global _titles_
   _titles_ [1] = val

def titlel (val) :
   global _titles_
   _titles_ [2] = val

def titler (val) :
   global _titles_
   _titles_ [3] = val

_mark_dict_ = {"dot": ".", "circle": "o", "cross": "x", "plus": "+", 
              "asterisk": "*"}

def attr ( *kw, ** keywords ) :
   """attr ( <keyword arguments> ) changes the values of certain
   keywords for the next graphics commands, until changed by
   keyword arguments to a plot command (changed for that command
   only), another attr command, or an nf (if ezcreset == "true",
   then attributes revert to their default values).
   If a keyword is a frame attribute (grid, scale, labels) then
   a new graph will be plotted regardless of the setting of _ezcshow_.
   """
   global _grid_type_, _axis_scales_, _thick_, _style_, _mark_, \
          _color_, _lev_, _krange_, _lrange_, _kstyle_, _lstyle_, \
          _region_, _labels_enabled_, _default_label_, _object_list_, \
          _null_object_list_, _win_on_
   if len (kw) > 0 :
      keywords = kw [0]
   if keywords.has_key ("grid") or keywords.has_key ("scale") or \
      keywords.has_key ("labels"):
      _nf_ = 1
   else :
      _nf_ = 0
   if keywords.has_key ("region") :
      _region_ = keywords ["region"]
   if keywords.has_key ("krange") :
      _krange_ = keywords ["krange"]
   if keywords.has_key ("lrange") :
      _lrange_ = keywords ["lrange"]
   if keywords.has_key ("kstyle") :
      _kstyle_ = keywords ["kstyle"]
   if keywords.has_key ("lstyle") :
      _lstyle_ = keywords ["lstyle"]
   if keywords.has_key ("grid"):
      _grid_type_ = keywords ["grid"]
      if _grid_type_ [0:2] == "no" :
         _grid_type_ = "none"
      elif _grid_type_ == "tickonly" :
         _grid_type_ = "axes"
      elif _grid_type_ == "xy" :
         _grid_type_ = "wide"
      else : # someday will implement x and y only
         _grid_type_ = "wide"
   if keywords.has_key ("scale"):
      _axis_scales_ = keywords ["scale"]
   if keywords.has_key ("thick"):
      _thick_ = keywords ["thick"]
   if keywords.has_key ("style"):
      _style_ = keywords ["style"]
   if keywords.has_key ("mark"):
      _mark_ = keywords ["mark"]
      if _mark_dict_.has_key (_mark_) :
         _mark_ = _mark_dict_ [_mark_]
   if keywords.has_key ("labels"):
      # Ignored if not either yes or no
      lbl = keywords ["labels"]
      if type (lbl) == StringType :
         if lbl [0] == 'y' or lbl [0] == 'Y' :
            _labels_enabled_ = 1
         elif lbl [0] == 'n' or lbl [0] == 'N' :
            _labels_enabled_ = 0
   if keywords.has_key ("label"):
      _default_label_ = keywords ["label"]
   if keywords.has_key ("color"):
      _color_ = keywords ["color"]
   if keywords.has_key ("lev") :
      _lev_ = keywords ["lev"]
   # Don't do the following if no windows are open yet, sf() will die.
   if _nf_ and _win_on_ != [0, 0, 0, 0, 0, 0, 0, 0]:
      # a frame trait was changed; plot the graph.
      # all other attributes are active only for later objects
      # introduced into the graph.
      for i in _object_list_ :
         if i == [] :
            continue # (there is nothing to graph yet)
         for j in range (len (i)) :
            # either set or reset object labels
            if _labels_enabled_ == 1 :
               i [j].set (label = _label_list_ [j])
            else :
               i [j].set (label = " ")
      if _object_list_ != _null_object_list_ :
         sf ( )
 
_style_dict_ = {"solid": "solid", "dashed": "dash", "dotted" : "dot",
                "pm" : "solid", "none": "none", "dotdash" : "dashdot"}
      
def plot (y = None, x = None, ** keywords) :
   global _grid_type_, _axis_scales_, _style_, _label_list_, \
      _labels_enabled_, _default_label_, _object_list_, _style_dict_, \
      _mark_dict_, _color_, _thick_, _style_, _ezcshow_, _labels_temporarily_enabled_
   if y is None :
      attr (keywords)
      return
   if keywords.has_key ("window") :
      window = keywords ["window"]
   else :
      window = "all"
   window = _check_win_ (window, minmax = 1)
   mark = " "
   if keywords.has_key ("grid") or keywords.has_key ("scale") :
      # Force a newframe if either of these keywords is specified
      _nf_ = 1
   else :
      _nf_ = 0
   if keywords.has_key ("grid"):
      _grid_type_ = keywords ["grid"]
      if _grid_type_ [0:2] == "no" :
         _grid_type_ = "none"
      elif _grid_type_ == "tickonly" :
         _grid_type_ = "axes"
      elif _grid_type_ == "xy" :
         _grid_type_ = "wide"
      else : # someday will implement x and y only
         _grid_type_ = "wide"
   if keywords.has_key ("scale"):
      _axis_scales_ = keywords ["scale"]
   _labels_temporarily_enabled_ = _labels_enabled_
   if keywords.has_key ("labels") :
      lbl = keywords ["labels"]
      if type (lbl) == StringType :
         if lbl [0] == 'y' or lbl [0] == 'Y' :
            _labels_temporarily_enabled_ = 1
         elif lbl [0] == 'n' or lbl [0] == 'N' :
            _labels_temporarily_enabled_ = 0
   if keywords.has_key ("label") :
      lbl = keywords ["label"]
   else :
      lbl = _default_label_
   if keywords.has_key ("color") :
      col = keywords ["color"]
   else :
      col = _color_
   # if y is a vector of coordinates lbl, x, and col should be converted to
   # objects of the same size.
   if len (shape (y)) == 1 :
      if no_of_dims (lbl) == 1 :
         lbl = lbl [0]
      if no_of_dims (col) == 1 :
         col = col [0]
      if x is not None and no_of_dims (x) == 2 :
         x = x [0]
      no_of_coords = 1
   else :
      no_of_coords = shape (y) [0]
      if is_scalar (lbl) :
         lbl = [lbl] * no_of_coords
      elif len (lbl) < no_of_coords :
         lbl = lbl + [" "] * (no_of_coords - len (lbl))
      if is_scalar (col) :
         col = [col] * no_of_coords
      elif len (col) < no_of_coords :
         col = col + [_color_] * (no_of_coords - len (col))
      if x is None or type (x) == ArrayType and len (shape (x)) == 1 :
         x = [x] * no_of_coords
      elif shape (x) [0] < no_of_coords :
         x = x + [None] * no_of_coords - shape (x) [0]
   if keywords.has_key ("thick") :
      thk = keywords ["thick"]
   else :
      thk = _thick_
   if keywords.has_key ("style") :
      stl = keywords ["style"]
      if stl == "dotted" :
         mark = "."
         stl = "dot"
      elif _style_dict_.has_key (stl) :
         stl = _style_dict_ [stl]
      # non-blank mark forced "none"
      hid = 0
   else :
      if _style_dict_.has_key (_style_) :
         stl = _style_dict_ [_style_]
      else :
         stl = _style_
      hid = 0
   if keywords.has_key ("mark"):
      mark = keywords ["mark"]
      if _mark_dict_.has_key (mark) :
         mark = _mark_dict_ [mark]
      # non-blank mark forced "none"
      if mark != " " :
         hid = 0
         stl = "none"
   if (_labels_enabled_ == 1 or _labels_temporarily_enabled_) and mark == " " \
      and lbl != " " and no_of_coords == 1 and stl == "none":
      mark = lbl [0]
   for j in window:
      if no_of_coords == 1 :
         if mark == " "  and not _labels_enabled_ and not _labels_temporarily_enabled_:
            _object_list_ [j].append (Curve (y = y, x = x, color = col, # label = lbl,
               type = stl, width = thk, hide = hid))
         elif  mark == " "  and (_labels_enabled_ or _labels_temporarily_enabled_):
            _object_list_ [j].append (Curve (y = y, x = x, color = col, label = lbl,
               type = stl, marks = 1, width = thk, hide = hid))
         else :
            _object_list_ [j].append (Curve (y = y, x = x, color = col, label = lbl,
               type = stl, marks = 1, marker = mark, width = thk, hide = hid))
         _label_list_.append (lbl)
      else :
         for i in range (no_of_coords) :
            if mark == " " and not (_labels_enabled_ or _labels_temporarily_enabled_):
               _object_list_ [j].append (Curve (y = y [i], x = x [i], color = col [i], 
                   # label = lbl [i],
                   type = stl, width = thk, hide = hid))
            elif  mark == " "  and (_labels_enabled_ or _labels_temporarily_enabled_):
               _object_list_ [j].append (Curve (y = y [i], x = x [i], color = col [i],
                   label = lbl [i], type = stl, marks = 1, width = thk,
                   hide = hid))
            else :
               _object_list_ [j].append (Curve (y = y [i], x = x [i], color = col [i],
                   label = lbl [i], type = stl, marks = 1, marker = mark,
                   width = thk, hide = hid))
         _label_list_ = _label_list_ + lbl
   if _nf_ or _ezcshow_ == 'true' :
      sf ( window = window )

_bnd_dict_ = {"Yes" : 1 , "yes" : 1 , "YES" : 1 , "y" : 1 , "Y" : 1 ,
              "ON" : 1 , "On" : 1 , "on" : 1 , 1 : 1,
              "No" : 0 , "no" : 0 , "NO" : 0 , "n" : 0 , "N" : 0 ,
              "None" : 0 , "none" : 0 , "NONE" : 0 ,
              "OFF" : 0 , "off" : 0 , "Off" : 0 , 0 : 0 }

_SetMeshError_ = "SetMeshError"

def set_mesh ( ** keywords ) :
   """set_mesh (rt = <array1>, zt = <array2>, ireg = <array3>,
   ut = <array3>, vt = <array4> , pvar = <array5>, cindex = <array6>)
   defines a two-dimensional mesh for plotting. rt and zt are real,
   two-dimensional arrays of the same shape defining the mesh. ireg
   is a two-dimensional integer array defining which region of the
   mesh each quadrilateral in it belongs to. It should be the same
   shape as rt and zt, but the first row and first column are
   constrained to be 0. ut and vt are velocity components used to
   plot vector fields. pvar and cindex are mutually exclusive. pvar
   is a real array used to color a filled mesh, while cindex is a
   character array whose components specify an index into a color
   table. All arrays must be the same shape. Once set,
   these variables will define the mesh until the next set_mesh
   command.  Any variable undefined by set_mesh must be supplied
   as a keyword argument to a plot command or must have been
   previously defined by a set_mesh command. A variable defined by
   set_mesh can be overruled temporarily by a keyword argument to
   a plot command.  Variables set by set_mesh will remain defaults
   unless they are overruled by keyword arguments to plot commands
   or until they are changed permanently by another set_mesh or by
   a clear_mesh.
   """
   global _zt_, _rt_, _ut_, _vt_, _ireg_, _pvar_, _cindex_
   if keywords.has_key ("zt") :
      _zt_ = keywords ["zt"]
   else :
      _zt_ = None
   if keywords.has_key ("rt") :
      _rt_ = keywords ["rt"]
   else :
      _rt_ = None
   if keywords.has_key ("ut") :
      _ut_ = keywords ["ut"]
   else :
      _ut_ = None
   if keywords.has_key ("vt") :
      _vt_ = keywords ["vt"]
   else :
      _vt_ = None
   if keywords.has_key ("ireg") :
      _ireg_ = keywords ["ireg"]
   else :
      _ireg_ = None
   if keywords.has_key ("pvar") and keywords.has_key ("cindex") :
      raise _SetMeshError_, "Only one of 'pvar' and 'cindex' is allowed."
   if keywords.has_key ("pvar") :
      _pvar_ = keywords ["pvar"]
      _cindex_ = None
   elif keywords.has_key ("cindex") :
      _cindex_ = keywords ["cindex"]
      _pvar_ = None

def clear_mesh ( ) :
   global _zt_, _rt_, _ireg_, _pvar_, _cindex_,  _ut_, _vt_
   _zt_ = None
   _rt_ = None
   _ireg_ = None
   _pvar_ = None
   _cindex_ = None
   _ut_ = None
   _vt_ = None

def ezcx (val) :
   global _zt_
   _zt_ = val

def ezcy (val) :
   global _rt_
   _rt_ = val

def ezcpvar (val) :
   global _pvar_
   _pvar_ = val

def ezccindex (val) :
   global _cindex_
   _cindex_ = val

def ezcireg (val) :
   global _ireg_
   _ireg_ = val

def ezcu (val ) :
   global _ut_
   _ut_ = val

def ezcv (val ) :
   global _vt_
   _vt_ = val

_MeshDefError_ = "MeshDefError"
_FillError_ = "FillError"

def _minm_ (x) :
   """_minm_ (x) computes and returns something guaranteed smaller than
   the minimum element in the array x."""
   min_ = min (ravel (x))
   if min_ == 0 :
      return -1
   elif min_ > 0 :
      return min_ - .01 * min_
   else :
      return min_ + .01 * min_

_PlotfError_ = "PlotfError"
_RangeError_ = "RangeError"

def plotm (*kwds, ** keywords) :
   global _grid_type_, _axis_scales_, _cmin_, _cmax_, _krange_, _lrange_, \
          _region_, _color_bar_, _labels_enabled_, _zt_, _rt_, _ut_, _vt_, \
          _ireg_, _pvar_, _cindex_, _object_list_, _labels_temporarily_enabled_
   _labels_temporarily_enabled_ = _labels_enabled_
   if len (kwds) != 0 :
      keywords = kwds [0]
   if keywords.has_key ("window") :
      window = keywords ["window"]
   else :
      window = "all"
   window = _check_win_ (window, minmax = 1)
   if keywords.has_key ("color_bar") :
      _color_bar_ = keywords ["color_bar"]
   if keywords.has_key ("contour_plot") and keywords ["contour_plot"] != 0 :
      cp = 1
   else :
      cp = 0
   if keywords.has_key ("lev") :
      __lev_ = keywords ["lev"]
   else :
      __lev_ = _lev_
   if keywords.has_key ("fill") :
      if _bnd_dict_.has_key (keywords ["fill"]) :
         _fill_ = _bnd_dict_ [keywords ["fill"]]
      else :
         raise _FillError_, "Unknown value <" + `keywords ["fill"]` + \
            "> for keyword 'fill.'"
   else :
      _fill_ = 0
   if keywords.has_key ("zt") :
      __zt_ = keywords ["zt"]
   else :
      __zt_ = _zt_
   if keywords.has_key ("rt") :
      __rt_ = keywords ["rt"]
   else :
      __rt_ = _rt_
   if keywords.has_key ("ut") :
      __ut_ = keywords ["ut"]
   else:
      __ut_ = _ut_
   if keywords.has_key ("vt") :
      __vt_ = keywords ["vt"]
   else:
      __vt_ = _vt_
   if keywords.has_key ("ireg") :
      __ireg_ = keywords ["ireg"]
   else :
      __ireg_ = _ireg_
   if __zt_ is None and __rt_ is not None and cp == 0 :
      raise _MeshDefError_, "zt is missing from the mesh.\n" + \
         "Use set_mesh command or zt keyword in plotm command."
   elif __zt_ is not None and __rt_ is None and cp == 0 :
      raise _MeshDefError_, "rt is missing from the mesh.\n" + \
         "Use set_mesh command or rt keyword in plotm command."
   elif __zt_ is None and __rt_ is None and cp == 0 :
      raise _MeshDefError_, "Both zt and rt are missing from the mesh.\n" + \
         "Use set_mesh command or rt and zt keywords in plotm command."
   if _fill_ == 0 and cp == 0 :
      __pvar_ = _pvar_
      __cindex_ = _cindex_
   if _fill_ == 1 or cp == 1 : # look for either pvar or cindex.
      __pvar_ = None
      __cindex_ = None
      if keywords.has_key ("pvar") :
         __pvar_ = keywords ["pvar"]
      elif keywords.has_key ("cindex") :
         __cindex_ = keywords ["cindex"]
      else :
         __pvar_ = _pvar_
         __cindex_ = _cindex_
      if __pvar_ is not None :
         k = shape (__pvar_) [0]
         l = shape (__pvar_) [1]
      elif __cindex_ is not None :
         k = shape (__cindex_) [0]
         l = shape (__cindex_) [1]
      else :
         raise _MeshDefError_, "Neither pvar nor cindex has been specified."
   if __ireg_ is None and __zt_ is not None : # Default to a single region
      if _fill_ == 0 :
         k = shape (__zt_) [0]
         l = shape (__zt_) [1]
      __ireg_ = ones ( (k, l), Int)
      __ireg_ [:, 0] = 0
      __ireg_ [0, :] = 0
   _z_ = None

   __krange_ = _krange_
   __lrange_ = _lrange_
   if keywords.has_key ("krange"):
      __krange_ = keywords ["krange"]
      if len (__krange_) >= 1 :
         klo = __krange_ [0]
         if klo < 0 or klo > shape (__ireg_) [0] :
            raise _RangeError_, `klo` + " is an illegal low subscript."
         if len (__krange_) >= 2 :
            khi = __krange_ [1]
            if khi < 0 or khi > shape (__ireg_) [0] :
               raise _RangeError_, `khi` + " is an illegal high subscript."
            if len (__krange_) >= 3 :
               kstr = __krange_ [2]
               if kstr < 0 or kstr > shape (__ireg_) [0] :
                  raise _RangeError_, `kstr` + " is an illegal stride."
            else :
               kstr = 1
         else :
            khi = shape (__ireg_) [0]
      else :
         raise _RangeError_, "<" + `__krange_` + "> is an illegal range."
   if keywords.has_key ("lrange"):
      __lrange_ = keywords ["lrange"]
      if len (__lrange_) >= 1 :
         llo = __lrange_ [0]
         if llo < 0 or llo > shape (__ireg_) [1] :
            raise _RangeError_, `llo` + " is an illegal low subscript."
         if len (__lrange_) >= 2 :
            lhi = __lrange_ [1]
            if lhi < 0 or lhi > shape (__ireg_) [1] :
               raise _RangeError_, `lhi` + " is an illegal high subscript."
            if len (__lrange_) >= 3 :
               lstr = __lrange_ [2]
               if lstr < 0 or lstr > shape (__ireg_) [1] :
                  raise _RangeError_, `lstr` + " is an illegal stride."
            else :
               lstr = 1
         else :
            lhi = shape (__ireg_) [1]
      else :
         raise _RangeError_, "<" + `__lrange_` + "> is an illegal range."
   if __krange_ is not None and __lrange_ is None :
      llo = 0
      lhi = shape (__ireg_) [1]
      lstr = 1
      __lrange_ = (llo, lhi, lstr)
   elif __krange_ is None and __lrange_ is not None :
      klo = 0
      khi = shape (__ireg_) [0]
      kstr = 1
      __krange_ = (klo, khi, kstr)
   # These attributes are sticky, so set the globals
   _krange_ = __krange_
   _lrange_ = __lrange_

   if __krange_ is not None : 
      # All the data must be resized
      __ireg_ = __ireg_ [klo:khi + 1:kstr, llo:lhi + 1:lstr]
      if __zt_ is not None :
         __zt_ = __zt_ [klo:khi + 1:kstr, llo:lhi + 1:lstr]
      if __rt_ is not None :
         __rt_ = __rt_ [klo:khi + 1:kstr, llo:lhi + 1:lstr]
      if __ut_ is not None :
         __ut_ = __ut_ [klo:khi + 1:kstr, llo:lhi + 1:lstr]
         __vt_ = __vt_ [klo:khi + 1:kstr, llo:lhi + 1:lstr]
      if __pvar_ is not None :
         __pvar_ = __pvar_ [klo:khi + 1:kstr, llo:lhi + 1:lstr]
      elif __cindex_ is not None:
         __cindex_ = __cindex_ [klo:khi + 1:kstr, llo:lhi + 1:lstr]

   _cscale_ = None
   if _fill_ == 1 or cp == 1 : # look for either pvar or cindex.
      if keywords.has_key ("cscale") :
         _cscale_ = keywords ["cscale"]
      else :
         _cscale_ = "lin"
      if _cscale_ != "lin" and __pvar_ is None :
         raise _PlotfError_, \
            "pvar keyword must be present if cscale is not 'lin.'"
      if __pvar_ is not None :
         _z_ = __pvar_
      elif __cindex_ is not None :
         _z_ = __cindex_
      if keywords.has_key ("edges") :
         _edges_ = keywords ["edges"]
         if keywords.has_key ("ewidth") :
            _ewidth_ = keywords ["ewidth"]
         else :
            _ewidth_ = 1
         if keywords.has_key ("ecolor") :
            _ecolor_ = keywords ["ecolor"]
         else :
            _ecolor_ = "fg"
      else :
         _edges_ = 0
         _ewidth_ = 0
         _ecolor_ = "bg"
   
   mark = " "
   if keywords.has_key ("grid") or keywords.has_key ("scale") :
      # Force a newframe if either of these keywords is specified
      _nf_ = 1
   else :
      _nf_ = 0
   if keywords.has_key ("grid"):
      _grid_type_ = keywords ["grid"]
      if _grid_type_ [0:2] == "no" :
         _grid_type_ = "none"
      elif _grid_type_ == "tickonly" :
         _grid_type_ = "axes"
      elif _grid_type_ == "xy" :
         _grid_type_ = "wide"
      else : # someday will implement x and y only
         _grid_type_ = "wide"
   if keywords.has_key ("scale"):
      _axis_scales_ = keywords ["scale"]
   else :
      _axis_scales_ = "linlin"
   if keywords.has_key ("style") :
      stl = keywords ["style"]
      if stl == "dotted" :
         mark = "."
      elif _style_dict_.has_key (stl) :
         stl = _style_dict_ [stl]
      hid = stl == "none"
   else :
      stl = "solid"
      hid = 0
   if keywords.has_key ("bnd") and _bnd_dict_.has_key (keywords ["bnd"]) :
      bnd = _bnd_dict_ [keywords ["bnd"]]
   else :
      bnd = 0
   # default kstyle and lstyle depend on setting of bnd.
   if bnd == 1 :
      _kstyle_ = "none"
      _lstyle_ = "none"
   else :
      _kstyle_ = stl
      _lstyle_ = stl
   if keywords.has_key ("kstyle"):
      _kstyle_ = keywords ["kstyle"]
      if _style_dict_.has_key (_kstyle_) :
         _kstyle_ = _style_dict_ [_kstyle_]
   if keywords.has_key ("lstyle"):
      _lstyle_ = keywords ["lstyle"]
      if _style_dict_.has_key (_lstyle_) :
         _lstyle_ = _style_dict_ [_lstyle_]
   _inhibit_ = 0
   if _kstyle_ == "none" :
      _inhibit_ = _inhibit_ + 1
   if _lstyle_ == "none" :
      _inhibit_ = _inhibit_ + 2
   if keywords.has_key ("thick") :
      thk = keywords ["thick"]
   else :
      thk = _thick_
   if keywords.has_key ("color") :
      col = keywords ["color"]
   else :
      col = _color_
   if keywords.has_key ("labels") :
      lbl = keywords ["labels"]
   else :
      lbl = " "
   if keywords.has_key ("mark"):
      _mark_ = keywords ["mark"]
   else :
      _mark_ = " "
   if keywords.has_key ("marksize"):
      _marksize_ = keywords ["marksize"]
   if keywords.has_key ("region"):
      __region_ = keywords ["region"]
      _region_ = __region_             # sticky
   else :
      __region_ = _region_
   if keywords.has_key ("vsc"):
      __vsc_ = keywords ["vsc"]
   else :
      __vsc_ = _vsc_
   if is_scalar (__region_) and __region_ != "all" :
      __region_ = [__region_]
   elif __region_ == "all" and bnd == 1 :
      # We need to make up a list of valid region numbers.
      # This is only for the boundary case--Gist only does
      # the outside boundary if region == 0, contrary to its
      # specifications.
      __region_ = []
      n = shape (__ireg_) [0]
      m = shape (__ireg_) [1]
      for i in range (n) :
         for j in range (m) :
            if __ireg_ [i,j] != 0 and not __ireg_ [i, j] in __region_ :
               __region_.append (__ireg_ [i, j])
   filled = 0
   contours = 0
   if cp == 1 and col [0:4] == "fill" :
      contours = filled = 1
      col = "fg"
   elif cp == 1 and col == "fillnl" :
      filled = 1
      col = "fg"
   elif cp == 1 :
      contours = 1
   if cp == 1 :
      _fill_ = filled
   if __region_ != "all" :
      # Plot the indicated regions
      _region_list_ = []
      for i in range (len (__region_)) :
         if _mark_ == " " and _labels_enabled_ == 0 :
            _region_list_.append ( Region (number = __region_ [i],
               boundary = bnd, boundary_type = stl, boundary_color = col,
               inhibit = _inhibit_, type = stl, color = col,
               ktype = _kstyle_, ltype = _lstyle_, filled = _fill_ ,
               width = thk, label = lbl, hide = hid, contours = contours,
               levels = __lev_))
         elif _mark_ == " " and _labels_enabled_ == 1 :
            _region_list_.append ( Region (number = __region_ [i],
               boundary = bnd, boundary_type = stl, boundary_color = col,
               inhibit = _inhibit_, type = stl, color = col,
               width = thk, label = lbl, hide = hid, marks = 1,
               ktype = _kstyle_, ltype = _lstyle_, filled = _fill_ ,
               contours = contours, levels = __lev_))
         else :
            _region_list_.append ( Region (number = __region_ [i],
               boundary = bnd, boundary_type = stl, boundary_color = col,
               inhibit = _inhibit_, type = stl, color = col,
               width = thk, label = lbl, hide = hid, marks = 1,
               ktype = _kstyle_, ltype = _lstyle_, filled = _fill_ ,
               marker = _mark_, contours = contours,
               levels = __lev_))
      for j in window :
         _object_list_ [j].append (QuadMesh (x = __zt_, y = __rt_, ireg = __ireg_,
            vx = __vt_, vy = __ut_, scale = __vsc_, z_scale = _cscale_,
            regions = _region_list_, filled = _fill_, contours = contours,
            levels = __lev_, z = _z_))
   else :
      # Plot the whole mesh
      for j in window :
         if _mark_ == " " and _labels_enabled_ == 0 :
            _object_list_ [j].append (QuadMesh (x = __zt_, y = __rt_, ireg = __ireg_,
               vx = __vt_, vy = __ut_, scale = __vsc_, regions = __region_,
               boundary = bnd, boundary_type = stl, boundary_color = col,
               inhibit = _inhibit_, type = stl, color = col,
               ktype = _kstyle_, ltype = _lstyle_, filled = _fill_ ,
               width = thk, label = lbl, hide = hid, z = _z_,
               contours = contours, levels = __lev_, z_scale = _cscale_))
         elif _mark_ == " " and _labels_enabled_ == 1 :
            _object_list_ [j].append (QuadMesh (x = __zt_, y = __rt_, ireg = __ireg_,
               vx = __vt_, vy = __ut_, scale = __vsc_, regions = __region_,
               boundary = bnd, boundary_type = stl, boundary_color = col,
               inhibit = _inhibit_, type = stl, color = col, z_scale = _cscale_,
               width = thk, label = lbl, hide = hid, marks = 1,
               ktype = _kstyle_, ltype = _lstyle_, filled = _fill_ ,
               z = _z_, contours = contours, levels = __lev_))
         else :
            _object_list_ [j].append (QuadMesh (x = __zt_, y = __rt_, ireg = __ireg_,
               vx = __vt_, vy = __ut_, scale = __vsc_, regions = __region_,
               boundary = bnd, boundary_type = stl, boundary_color = col,
               inhibit = _inhibit_, type = stl, color = col, z_scale = _cscale_,
               width = thk, label = lbl, hide = hid, marks = 1,
               ktype = _kstyle_, ltype = _lstyle_, filled = _fill_ ,
               marker = _mark_, z = _z_, contours = contours, levels = __lev_))
   _label_list_.append (lbl)
   if _nf_ or _ezcshow_ == 'true' :
      sf ( window = window )

_PlotvError_ = "PlotvError"

def plotv (*args, **keywords) :
   """plotv ( [zt [, rt [, vt [,ut [, ireg]]]]], <keyword arguments> )
   plots vectors on a mesh."""
   rt = _rt_
   zt = _zt_
   ut = _ut_
   vt = _vt_
   ireg = _ireg_
   if not keywords.has_key ("kstyle") :
      keywords ["kstyle"] = "none"
   if not keywords.has_key ("lstyle") :
      keywords ["lstyle"] = "none"
   if len (args) > 0 :
      zt = args [0]
      keywords ["zt"] = zt
      if len (args) > 1 :
         rt = args [1]
         keywords ["rt"] = rt
         if len (args) > 2 :
            vt = args [2]
            keywords ["vt"] = vt
            if len (args) > 3 :
               ut = args [3]
               keywords ["ut"] = ut
               if len (args) > 4 :
                  ireg = args [4]
                  keywords ["ireg"] = ireg
               elif keywords.has_key ("ireg") :
                  ireg = keywords ["ireg"]
            elif keywords.has_key ("ut") :
               ut = keywords ["ut"]
         elif keywords.has_key ("vt") :
            vt = keywords ["vt"]
      elif keywords.has_key ("rt") :
         rt = keywords ["rt"]
   else :
      if keywords.has_key ("zt") :
         zt = keywords ["zt"]
      if keywords.has_key ("rt") :
         rt = keywords ["rt"]
      if keywords.has_key ("ut") :
         ut = keywords ["ut"]
      if keywords.has_key ("vt") :
         vt = keywords ["vt"]
   if rt is None :
      raise _PlotvError_, "rt has not been defined. Use set_mesh, or pass\n" + \
         "as second argument or keyword argument to plotv."
   if zt is None :
      raise _PlotvError_, "zt has not been defined. Use set_mesh, or pass\n" + \
         "as first argument or keyword argument to plotv."
   if ut is None :
      raise _PlotvError_, "ut has not been defined. Use set_mesh, or pass\n" + \
         "as fourth argument or keyword argument to plotv."
   if vt is None :
      raise _PlotvError_, "vt has not been defined. Use set_mesh, or pass\n" + \
         "as third argument or keyword argument to plotv."
   plotm ( keywords )

def plotb (** keywords) :
   """plotb ( <keyword arguments> ) is the same as plotm with the
   keyword bnd = 1, i. e., plot the boundary of the mesh. """
   keywords ["bnd"] = 1
   plotm ( keywords )

_PlotcError_ = "PlotcError"

def plotc (** keywords) :
   """plotc ( <keyword arguments> ) is similar to plotm; however, it
   adds to the keywords in such a way that plotm will do a contour plot.
   As in plotf, zt and rt may have been specified as one dimensional,
   and if so, they need to be expanded to two dimensions.
   """
   global _lev_, _pvar_
   if keywords.has_key ("pvar") :
      (k, l) = shape (keywords ["pvar"])
   elif _pvar_ is not None :
      (k, l) = shape (_pvar_)
   else :
      raise _PlotcError_ , "pvar keyword must be specified or " + \
         "pvar must be defined by set_mesh."
   if keywords.has_key ("zt") :
      x = keywords ["zt"]
      if no_of_dims (x) == 1 :
         if len (x)  != k :
            raise _PlotcError_, "Length of zt <" + `len (x)` + \
               "> does not match first dimension of pvar <" + `k` + ">."
         keywords ["zt"] = multiply.outer (x, ones (l))
      elif shape (y) != (k, l) :
         raise _PlotcError_, "zt has a funny shape: " + `shape (y)` + \
            "."
   if keywords.has_key ("rt") :
      y = keywords ["rt"]
      if no_of_dims (y) == 1 :
         if len (y)  != l : 
            raise _PlotcError_, "Length of rt <" + `len (y)` + \
               "> does not match first dimension of pvar <" + `l` + ">."
         keywords ["rt"] = multiply.outer (ones (k), y)
      elif shape (x) != (k, l) :
         raise _PlotcError_, "rt has a funny shape: " + `shape (x)` + \
            "."
   keywords ["kstyle"] = "none"
   keywords ["lstyle"] = "none"
   keywords ["contour_plot"] = 1
   if keywords.has_key ("lev") :
      lev = keywords ["lev"]
   else :
      lev = _lev_
   if lev == "lin" or lev == "linear" :
      lev = deflev
   if lev == "log" or type (lev) == IntType and lev < 0 :
      keywords ["cscale"] = "log"
      if lev == "log" :
         lev = deflev
      else :
         lev = - lev
   keywords ["lev"] = lev
   plotm (keywords)

_PlotzError_ = "PlotzError"

def plotz (* args, **keywords) :
   """plotz (z [,x [,y]], <keyword arguments> ) is the non mesh-oriented
   contour plot. keyword pvar will be set to z, rt will be set to x
   (if present), and zt will be set to y (if present). If x or y or
   both are 1d, they will be expanded to a 2d mesh. This is so that
   plotm can be used."""
   global _lev_
   keywords ["kstyle"] = "none"
   keywords ["lstyle"] = "none"
   keywords ["contour_plot"] = 1
   if len (args) != 0 :
      z = args [0]
   elif keywords.has_key ("z") :
      z = keywords ["z"]
      del keywords ["z"]
   else :
      raise _PlotzError_, "No z argument was given for plotz!!"
   if len (args) > 1 :
      x = args [1]
      if len (args) > 2 :
         y = args [2]
      else :
         y = None
   else :
      x = None
      y = None
   if x is not None and y is not None and \
      len (x.shape) == len (y.shape) == len (z.shape) == 1 and \
      x.shape == y.shape == z.shape:
      # random data; turn into a regular mesh.
      xmax = max (x)
      xmin = min (x)
      ymax = max (y)
      ymin = min (y)
      nx = min (int (sqrt (len (x))) + 1, 50)
      ny = nx
      dx = (xmax - xmin) / nx
      dy = (ymax - ymin) / ny
      xcplot = span (xmin, xmax - dx, nx)
      ycplot = span (ymin, ymax - dy, ny)
      del xmin, xmax, ymin, ymax, dx, dy
      xt = subtract.outer (x, x)
      yt = subtract.outer (y, y)
      aa = sqrt (xt * xt + yt * yt + rsq)
      del xt, yt
      alpha = array (z, copy = 1)
      alpha = solve_linear_equations (aa, alpha)
      z = mfit (alpha, x, xcplot, y,
         ycplot, rsq)
      del aa, alpha, rsq
      # Expand coordinates to 2d arrays to match zcplot
      x = multiply.outer (xcplot, ones (ny, Float))
      y = multiply.outer (ones (nx, Float), ycplot)
      del xcplot, ycplot, nx, ny
   keywords ["pvar"] = z
   (k, l) = shape (z)
   if x is not None and len (shape (x)) == 1 :
      if len (x) != k :
         raise _PlotzError_, "length of x <" + `len (x)` + \
           "> does not match first dimension of z <" + `k` + ">."
      x = multiply.outer (x, ones (l, Float))
   elif x is None :
      x = multiply.outer (arange (1, k + 1, typecode = Float), ones (l, Float))
   if y is not None and len (shape (y)) == 1 :
      if len (y) != l :
         raise _PlotzError_, "length of y <" + `len (y)` + \
           "> does not match second dimension of z <" + `l` + ">."
      y = multiply.outer (ones (k, Float), y)
   elif y is None :
      y = multiply.outer (ones (k, Float), arange (1, l + 1, typecode = Float))
   keywords ["zt"] = y
   keywords ["rt"] = x
   if keywords.has_key ("lev") :
      lev = keywords ["lev"]
   else :
      lev = _lev_
   if lev == "lin" or lev == "linear" :
      lev = deflev
   if lev == "log" or type (lev) == IntType and lev < 0 :
      keywords ["cscale"] = "log"
      if lev == "log" :
         lev = deflev
      else :
         lev = - lev
   keywords ["lev"] = lev
   plotm (keywords)

_PlotiError_ = "PlotiError"

def ploti (* pvar, ** keywords) :
   """ploti (pvar, <keyword arguments> ) is the cell array plot."""
   global _object_list_, _ezcshow_, _pvar_, _labels_enabled_, \
      _labels_temporarily_enabled_
   _labels_temporarily_enabled_ = _labels_enabled_
   if _object_list_ != _null_object_list_ :
      raise _PlotiError_, "Cell array plot can't be superposed with others."
   if len (pvar) > 0 :
      pvar = pvar [0]
   elif keywords.has_key ("pvar") :
      pvar = keywords ["pvar"]
   elif _pvar_ is not None :
      pvar = _pvar_
   else :
      pvar = _cindex_
   if keywords.has_key ("window") :
      window = keywords ["window"]
   else :
      window = "all"
   window = _check_win_ (window, minmax = 1)
   if keywords.has_key ("style") :
      hid = keywords ["style"] == "none"
   else :
      hid = 0
   if keywords.has_key ("labels") :
      lbl = keywords ["labels"]
   else :
      lbl = " "
   for j in window:
      _object_list_ [j].append ( CellArray (z = pvar, hide = hid, label = lbl))
   if _ezcshow_ == 'true' :
      sf ( window = window )

def plotf (*args, ** keywords) :
   """plotf ( arg1 [,arg2 [,arg3 [,arg4 ]]] [,<keyword arguments>] )
   is the same as plotm with the keyword fill = 1, i. e., plot a
   filled mesh. arg1 if integer is an array which gives color
   numbers; if real, gives the values of a real variable which
   will be used to interpolate into the current palette. arg2,
   if present, is zt. arg3, if present, is rt. arg4, if present,
   is ireg."""
   global _zt_, _rt_, _pvar_
   n = len (args)
   __zt_ = _zt_
   __rt_ = _rt_
   if n != 0 :
      if type (args[0][0][0]) == IntType :
         plvar = keywords ["cindex"] = array(args [0],'b') # Gist knows chars
      else :
         plvar = keywords ["pvar"] = args [0]
         if n > 1 :
            __zt_ = keywords ["zt"] = args [1]
         if n > 2 :
            __rt_ = keywords ["rt"] = args [2]
         if n > 3 :
            keywords ["ireg"] = args [3]
   elif keywords is not None and not keywords.has_key ("cindex") \
      and not keywords.has_key ("pvar") and _pvar_ is None :
      raise _PlotfError_, "plotf requires at least one non-keyword " + \
         "argument\n or else one of the keywords 'cindex' or 'pvar.'"
   elif not keywords.has_key ("cindex") and not keywords.has_key ("pvar") \
        and _pvar_ is not None :
      plvar = keywords ["pvar"] = _pvar_
   elif keywords.has_key ("cindex") :
      plvar = keywords ["cindex"]
   else :
      plvar = keywords ["pvar"]
   # zt and rt might not be specified, or else thay are vectors.
   # They must be 2d and match plvar in shape.
   (k, l) = shape (plvar)
   if shape (__zt_) != shape (plvar) :
      if __zt_ is None:
         __zt_ = multiply.outer (arange (1, k + 1, typecode = Float), ones (l, Float))
      elif no_of_dims (__zt_) == 1 :
         if len (__zt_) != k :
            raise _PlotfError_, "Length of zt <" + `len (__zt_)` + \
               "> does not match first dimension of pvar <" + `k` + ">."
         __zt_ = multiply.outer (__zt_, ones (l, Float))
      else :
         raise _PlotfError_, "zt has a funny shape: " + `shape (__zt_)` + \
            "."
      keywords ["zt"] = __zt_
   if shape (__rt_) != shape (plvar) :
      if __rt_ is None:
         __rt_ = multiply.outer (ones (k, Float), arange (1, l + 1, typecode = Float))
      elif no_of_dims (__rt_) == 1 :
         if len (__rt_) != l :
            raise _PlotfError_, "Length of rt <" + `len (__rt_)` + \
               "> does not match second dimension of pvar <" + `l` + ">."
         __rt_ = multiply.outer (ones (k, Float), __rt_)
      else :
         raise _PlotfError_, "rt has a funny shape: " + `shape (__rt_)` + \
            "."
      keywords ["rt"] = __rt_
   keywords ["fill"] = 1
   plotm ( keywords )

def frame ( *args , ** keywords ) :
   """frame ( [xmin [, xmax [, ymin [, ymax]]]] [, <keyword srguments> )
   has the effect of changing the axis limits of the current graph.
   The limits may be expressed by positional arguments or by keyword
   arguments. The limits are effective immediately on the current
   graph and disappear on the next one. Any limits not set by this
   command will default to the maximum and the minimum of the data.
   """
   global _axis_limits_
   if keywords.has_key ("window") :
      window = keywords ["window"]
   else :
      window = "all"
   window = _check_win_ (window, minmax = 1)
   if keywords.has_key ("xmin") :
      _axis_limits_ [0] [0] = keywords ["xmin"]
   elif len (args) > 0 :
      _axis_limits_ [0] [0] = args [0]
   else :
      _axis_limits_ [0] [0] = "d"
   if keywords.has_key ("xmax") :
      _axis_limits_ [0] [1] = keywords ["xmax"]
   elif len (args) > 1 :
      _axis_limits_ [0] [1] = args [1]
   else :
      _axis_limits_ [0] [1] = "d"
   if keywords.has_key ("ymin") :
      _axis_limits_ [1] [0] = keywords ["ymin"]
   elif len (args) > 2 :
      _axis_limits_ [1] [0] = args [2]
   else :
      _axis_limits_ [1] [0] = "d"
   if keywords.has_key ("ymax") :
      _axis_limits_ [1] [1] = keywords ["ymax"]
   elif len (args) > 3 :
      _axis_limits_ [1] [1] = args [3]
   else :
      _axis_limits_ [1] [1] = "d"
   sf ( window = window )

def fr ( *args , ** keywords ) :
   """fr ( [xmin [, xmax [, ymin [, ymax]]]] [, <keyword srguments> )
   is the equivalent of nf ( ) followed by frame with the same
   set of arguments."""
   global _axis_limits_
   if keywords.has_key ("window") :
      window = keywords ["window"]
   else :
      window = "all"
   window = _check_win_ (window, minmax = 1)
   nf ( window = window )
   if keywords.has_key ("xmin") :
      _axis_limits_ [0] [0] = keywords ["xmin"]
   elif len (args) > 0 :
      _axis_limits_ [0] [0] = args [0]
   else :
      _axis_limits_ [0] [0] = "d"
   if keywords.has_key ("xmax") :
      _axis_limits_ [0] [1] = keywords ["xmax"]
   elif len (args) > 1 :
      _axis_limits_ [0] [1] = args [1]
   else :
      _axis_limits_ [0] [1] = "d"
   if keywords.has_key ("ymin") :
      _axis_limits_ [1] [0] = keywords ["ymin"]
   elif len (args) > 2 :
      _axis_limits_ [1] [0] = args [2]
   else :
      _axis_limits_ [1] [0] = "d"
   if keywords.has_key ("ymax") :
      _axis_limits_ [1] [1] = keywords ["ymax"]
   elif len (args) > 3 :
      _axis_limits_ [1] [1] = args [3]
   else :
      _axis_limits_ [1] [1] = "d"
   sf ( window = window )

def text (str, rt, zt, size, tosys = 1, color = "fg", window = "all") :
   """text (str, zt, rt, size [, tosys]) plots the specified string starting at
   (zt, rt) in point size 'size.' If tosys is specified and nonzero,
   then coordinates are not window-relative."""
   global _text_, _text_pos_, _text_color_, _text_size_, _tosys_, _ezcshow_
   _text_.append (str)
   _text_pos_.append ([rt, zt])
   _text_color_.append (color)
   _text_size_.append (size)
   _tosys_.append (tosys)
   if _ezcshow_ == 'true' :
      sf ( window = window )

def list_devices ( ) :
   global _win_on_, _win_plotters_, _cgm_file_, _cgm_, _ps_file_, _ps_, \
      _cgm_plotter_, _ps_plotter_, _graphics_, _displays_
   if _win_on_ == [0, 0, 0, 0, 0, 0, 0, 0] :
      print "No devices are currently open."
      return
   for i in range (len (_win_plotters_) ) :
      if i == _cgm_plotter_ :
         if _cgm_ == "yes" :
            op = "on"
         else :
            op = "off"
         print "device %d: cgm file '%s', currently %s." % \
            (i, _cgm_file_, op)
      elif i == _ps_plotter_ :
         if _ps_ == "yes" :
            op = "on"
         else :
            op = "off"
         print "device %d: ps file '%s', currently %s." % \
            (i, _ps_file_, op)
      elif _win_on_ [i] :
         if _graphics_ [i] [0:3] == "Nar" or Graphics == NarPlotter :
            gr = "Narcisse"
         elif _graphics_ [i] == "Gist" or Graphics == GistPlotter :
            gr = "Gist"
         else :
            gr = Graphics
         print "device %d: %s Xwindow for display '%s'." % \
            (i, gr, _displays_ [i])
   return
